@@ -40,8 +40,10 @@
 #include <qlayout.h>
 #include <qlabel.h>
 
+//BEGIN KateSearch
 QStringList KateSearch::s_searchList  = QStringList();
 QStringList KateSearch::s_replaceList = QStringList();
+QString KateSearch::s_pattern = QString();
 static const bool arbitraryHLExample = false;
 
 KateSearch::KateSearch( KateView* view )
@@ -100,40 +102,52 @@ void KateSearch::find()
 
   if( findDialog->exec() == QDialog::Accepted ) {
     s_searchList =  findDialog->findHistory () ;
-    KateViewConfig::global()->setSearchFlags(findDialog->options ());
-
-    SearchFlags searchFlags;
-
-    searchFlags.caseSensitive = KateViewConfig::global()->searchFlags() & KFindDialog::CaseSensitive;
-    searchFlags.wholeWords = KateViewConfig::global()->searchFlags() & KFindDialog::WholeWordsOnly;
-    searchFlags.fromBeginning = !(KateViewConfig::global()->searchFlags() & KFindDialog::FromCursor)
-                               && !(KateViewConfig::global()->searchFlags() & KFindDialog::SelectedText);
-    searchFlags.backward = KateViewConfig::global()->searchFlags() & KFindDialog::FindBackwards;
-    searchFlags.selected = KateViewConfig::global()->searchFlags() & KFindDialog::SelectedText;
-    searchFlags.prompt = false;
-    searchFlags.replace = false;
-    searchFlags.finished = false;
-    searchFlags.regExp = KateViewConfig::global()->searchFlags() & KFindDialog::RegularExpression;
-
-    if ( searchFlags.selected )
-    {
-      s.selBegin = KateTextCursor( doc()->selStartLine(), doc()->selStartCol() );
-      s.selEnd   = KateTextCursor( doc()->selEndLine(),   doc()->selEndCol()   );
-      s.cursor   = s.flags.backward ? s.selEnd : s.selBegin;
-    } else {
-      s.cursor = getCursor();
-    }
-
-    s.wrappedEnd = s.cursor;
-    s.wrapped = false;
-
-    search( searchFlags );
+    // Do *not* remove the QString() wrapping, it fixes a nasty crash
+    find( QString(s_searchList.first()), findDialog->options(), true, true );
   }
 
   delete findDialog;
   m_view->repaintText ();
 }
 
+void KateSearch::find( const QString &pattern, long flags, bool add, bool shownotfound )
+{
+  KateViewConfig::global()->setSearchFlags( flags );
+  if( add )
+    addToList( s_searchList, pattern );
+
+   s_pattern = pattern;
+
+  SearchFlags searchFlags;
+
+  searchFlags.caseSensitive = KateViewConfig::global()->searchFlags() & KFindDialog::CaseSensitive;
+  searchFlags.wholeWords = KateViewConfig::global()->searchFlags() & KFindDialog::WholeWordsOnly;
+  searchFlags.fromBeginning = !(KateViewConfig::global()->searchFlags() & KFindDialog::FromCursor)
+      && !(KateViewConfig::global()->searchFlags() & KFindDialog::SelectedText);
+  searchFlags.backward = KateViewConfig::global()->searchFlags() & KFindDialog::FindBackwards;
+  searchFlags.selected = KateViewConfig::global()->searchFlags() & KFindDialog::SelectedText;
+  searchFlags.prompt = false;
+  searchFlags.replace = false;
+  searchFlags.finished = false;
+  searchFlags.regExp = KateViewConfig::global()->searchFlags() & KFindDialog::RegularExpression;
+  searchFlags.useBackRefs = KateViewConfig::global()->searchFlags() & KReplaceDialog::BackReference;
+
+  if ( searchFlags.selected )
+  {
+    s.selBegin = KateTextCursor( doc()->selStartLine(), doc()->selStartCol() );
+    s.selEnd   = KateTextCursor( doc()->selEndLine(),   doc()->selEndCol()   );
+    s.cursor   = s.flags.backward ? s.selEnd : s.selBegin;
+  } else {
+    s.cursor = getCursor();
+  }
+
+  s.wrappedEnd = s.cursor;
+  s.wrapped = false;
+  s.showNotFound = shownotfound;
+
+  search( searchFlags );
+}
+
 void KateSearch::replace()
 {
   if (!doc()->isReadWrite()) return;
@@ -149,41 +163,56 @@ void KateSearch::replace()
   replaceDialog->setPattern (getSearchText());
 
   if( replaceDialog->exec() == QDialog::Accepted ) {
+    long opts = replaceDialog->options();
     m_replacement = replaceDialog->replacement();
     s_searchList = replaceDialog->findHistory () ;
     s_replaceList = replaceDialog->replacementHistory () ;
-    KateViewConfig::global()->setSearchFlags(replaceDialog->options ());
 
-    SearchFlags searchFlags;
-    searchFlags.caseSensitive = KateViewConfig::global()->searchFlags() & KFindDialog::CaseSensitive;
-    searchFlags.wholeWords = KateViewConfig::global()->searchFlags() & KFindDialog::WholeWordsOnly;
-    searchFlags.fromBeginning = !(KateViewConfig::global()->searchFlags() & KFindDialog::FromCursor)
-                               && !(KateViewConfig::global()->searchFlags() & KFindDialog::SelectedText);
-    searchFlags.backward = KateViewConfig::global()->searchFlags() & KFindDialog::FindBackwards;
-    searchFlags.selected = KateViewConfig::global()->searchFlags() & KFindDialog::SelectedText;
-    searchFlags.prompt = KateViewConfig::global()->searchFlags() & KReplaceDialog::PromptOnReplace;
-    searchFlags.replace = true;
-    searchFlags.finished = false;
-    searchFlags.regExp = KateViewConfig::global()->searchFlags() & KFindDialog::RegularExpression;
-    if ( searchFlags.selected )
-    {
-      s.selBegin = KateTextCursor( doc()->selStartLine(), doc()->selStartCol() );
-      s.selEnd   = KateTextCursor( doc()->selEndLine(),   doc()->selEndCol()   );
-      s.cursor   = s.flags.backward ? s.selEnd : s.selBegin;
-    } else {
-      s.cursor = getCursor();
-    }
-
-    s.wrappedEnd = s.cursor;
-    s.wrapped = false;
-
-    search( searchFlags );
+    // Do *not* remove the QString() wrapping, it fixes a nasty crash
+    replace( QString(s_searchList.first()), m_replacement, opts );
   }
 
   delete replaceDialog;
   m_view->update ();
 }
 
+void KateSearch::replace( const QString& pattern, const QString &replacement, long flags )
+{
+  if (!doc()->isReadWrite()) return;
+
+  addToList( s_searchList, pattern );
+   s_pattern = pattern;
+  addToList( s_replaceList, replacement );
+  m_replacement = replacement;
+  KateViewConfig::global()->setSearchFlags( flags );
+
+  SearchFlags searchFlags;
+  searchFlags.caseSensitive = KateViewConfig::global()->searchFlags() & KFindDialog::CaseSensitive;
+  searchFlags.wholeWords = KateViewConfig::global()->searchFlags() & KFindDialog::WholeWordsOnly;
+  searchFlags.fromBeginning = !(KateViewConfig::global()->searchFlags() & KFindDialog::FromCursor)
+      && !(KateViewConfig::global()->searchFlags() & KFindDialog::SelectedText);
+  searchFlags.backward = KateViewConfig::global()->searchFlags() & KFindDialog::FindBackwards;
+  searchFlags.selected = KateViewConfig::global()->searchFlags() & KFindDialog::SelectedText;
+  searchFlags.prompt = KateViewConfig::global()->searchFlags() & KReplaceDialog::PromptOnReplace;
+  searchFlags.replace = true;
+  searchFlags.finished = false;
+  searchFlags.regExp = KateViewConfig::global()->searchFlags() & KFindDialog::RegularExpression;
+  searchFlags.useBackRefs = KateViewConfig::global()->searchFlags() & KReplaceDialog::BackReference;
+  if ( searchFlags.selected )
+  {
+    s.selBegin = KateTextCursor( doc()->selStartLine(), doc()->selStartCol() );
+    s.selEnd   = KateTextCursor( doc()->selEndLine(),   doc()->selEndCol()   );
+    s.cursor   = s.flags.backward ? s.selEnd : s.selBegin;
+  } else {
+    s.cursor = getCursor();
+  }
+
+  s.wrappedEnd = s.cursor;
+  s.wrapped = false;
+
+  search( searchFlags );
+}
+
 void KateSearch::findAgain( bool back )
 {
   SearchFlags searchFlags;
@@ -197,10 +226,11 @@ void KateSearch::findAgain( bool back )
   searchFlags.replace = false;
   searchFlags.finished = false;
   searchFlags.regExp = KateViewConfig::global()->searchFlags() & KFindDialog::RegularExpression;
+  searchFlags.useBackRefs = KateViewConfig::global()->searchFlags() & KReplaceDialog::BackReference;
 
   searchFlags.backward = searchFlags.backward != back;
   searchFlags.fromBeginning = false;
-  searchFlags.prompt = true;
+  searchFlags.prompt = true; // ### why is the above assignment there?
   s.cursor = getCursor();
 
   search( searchFlags );
@@ -251,7 +281,7 @@ void KateSearch::wrapSearch()
       s.cursor.setPos(0, 0);
     } else {
       s.cursor.setLine(doc()->numLines() - 1);
-      s.cursor.setCol(doc()->lineLength( s.cursor.line() ));
+      s.cursor.setCol(doc()->lineLength( s.cursor.line() ) );
     }
   }
 
@@ -265,14 +295,12 @@ void KateSearch::wrapSearch()
 
 void KateSearch::findAgain()
 {
-  QString searchFor = s_searchList.first();
-
-  if( searchFor.isEmpty() ) {
+  if(  s_pattern.isEmpty() ) {
     find();
     return;
   }
 
-  if ( doSearch( searchFor ) ) {
+  if ( doSearch(  s_pattern ) ) {
     exposeFound( s.cursor, s.matchedLength );
   } else if( !s.flags.finished ) {
     if( askContinue() ) {
@@ -283,20 +311,19 @@ void KateSearch::findAgain()
     }
   } else {
     if (arbitraryHLExample) m_arbitraryHLList->clear();
-    KMessageBox::sorry( view(),
+    if ( s.showNotFound )
+      KMessageBox::sorry( view(),
         i18n("Search string '%1' not found!")
-             .arg( KStringHandler::csqueeze( searchFor ) ),
+             .arg( KStringHandler::csqueeze(  s_pattern ) ),
         i18n("Find"));
   }
 }
 
 void KateSearch::replaceAll()
 {
-  QString searchFor = s_searchList.first();
-
   doc()->editStart ();
 
-  while( doSearch( searchFor ) )
+  while( doSearch(  s_pattern ) )
     replaceOne();
 
   doc()->editEnd ();
@@ -315,8 +342,7 @@ void KateSearch::replaceAll()
 
 void KateSearch::promptReplace()
 {
-  QString searchFor = s_searchList.first();
-  if ( doSearch( searchFor ) ) {
+  if ( doSearch(  s_pattern ) ) {
     exposeFound( s.cursor, s.matchedLength );
     replacePrompt->show();
     replacePrompt->setFocus ();
@@ -335,7 +361,7 @@ void KateSearch::promptReplace()
 void KateSearch::replaceOne()
 {
   QString replaceWith = m_replacement;
-  if ( s.flags.regExp ) {
+  if ( s.flags.regExp && s.flags.useBackRefs ) {
     // replace each "(?!\)\d+" with the corresponding capture
     QRegExp br("\\\\(\\d+)");
     int pos = br.search( replaceWith );
@@ -349,7 +375,6 @@ void KateSearch::replaceOne()
           replaceWith.replace( pos, br.matchedLength(), sc );
         }
         else {
-          // TODO add a sanity check at some point prior to this
           kdDebug()<<"KateSearch::replaceOne(): you don't have "<<ccap<<" backreferences in regexp '"<<m_re.pattern()<<"'"<<endl;
         }
       }
@@ -364,6 +389,7 @@ void KateSearch::replaceOne()
   doc()->editEnd(),
 
   replaces++;
+
   // if we inserted newlines, we better adjust.
   uint newlines = replaceWith.contains('\n');
   if ( newlines )
@@ -378,6 +404,7 @@ void KateSearch::replaceOne()
       s.selEnd.setLine( s.selEnd.line() + newlines );
   }
 
+
   // adjust selection endcursor if needed
   if( s.flags.selected && s.cursor.line() == s.selEnd.line() )
   {
@@ -542,6 +569,7 @@ bool KateSearch::doSearch( const QString& text )
   uint foundLine, foundCol, matchLen;
   bool found = false;
   //kdDebug() << "Searching at " << line << ", " << col << endl;
+//   kdDebug()<<"KateSearch::doSearch: "<<line<<", "<<col<<", "<<backward<<endl;
 
   do {
       if( regExp ) {
@@ -600,7 +628,7 @@ bool KateSearch::doSearch( const QString& text )
     }
   }
 
-  //kdDebug() << "Found at " << s.cursor.line() << ", " << s.cursor.col() << endl;
+//   kdDebug() << "Found at " << s.cursor.line() << ", " << s.cursor.col() << endl;
 
 
   //m_searchResults.append(s);
@@ -635,15 +663,16 @@ void KateSearch::exposeFound( KateTextCursor &cursor, int slen )
   view()->setCursorPositionInternal ( cursor.line(), cursor.col() + slen, 1 );
   doc()->setSelection( cursor.line(), cursor.col(), cursor.line(), cursor.col() + slen );
 }
+//END KateSearch
 
 //BEGIN KateReplacePrompt
 // this dialog is not modal
 KateReplacePrompt::KateReplacePrompt ( QWidget *parent )
   : KDialogBase ( parent, 0L, false, i18n( "Replace Confirmation" ),
                   User3 | User2 | User1 | Close | Ok , Ok, true,
-                  i18n("Replace &All"), i18n("Replace && Close"), i18n("&Replace") )
+                  i18n("Replace &All"), i18n("Re&place && Close"), i18n("&Replace") )
 {
-  setButtonOK( i18n("Find Next") );
+  setButtonOK( i18n("&Find Next") );
   QWidget *page = new QWidget(this);
   setMainWidget(page);
 
@@ -685,4 +714,212 @@ void KateReplacePrompt::done (int result)
 }
 //END KateReplacePrompt
 
+//BEGIN SearchCommand
+bool SearchCommand::exec(class Kate::View *view, const QString &cmd, QString &msg)
+{
+  QString flags, pattern, replacement;
+  if ( cmd.startsWith( "find" ) )
+  {
+
+    static QRegExp re_find("find(?::([bcersw]*))?\\s+(.+)");
+    if ( re_find.search( cmd ) < 0 )
+    {
+      msg = i18n("Usage: find[:[bcersw]] PATTERN");
+      return false;
+    }
+    flags = re_find.cap( 1 );
+    pattern = re_find.cap( 2 );
+  }
+
+  else if ( cmd.startsWith( "ifind" ) )
+  {
+    static QRegExp re_ifind("ifind(?::([bcrs]*))?\\s+(.*)");
+    if ( re_ifind.search( cmd ) < 0 )
+    {
+      msg = i18n("Usage: ifind[:[bcrs]] PATTERN");
+      return false;
+    }
+    ifindClear();
+    return true;
+  }
+
+  else if ( cmd.startsWith( "replace" ) )
+  {
+    // Try if the pattern and replacement is quoted, using a quote character ["']
+    static QRegExp re_rep("replace(?::([bceprsw]*))?\\s+([\"'])((?:[^\\\\\\\\2]|\\\\.)*)\\2\\s+\\2((?:[^\\\\\\\\2]|\\\\.)*)\\2\\s*$");
+    // Or one quoted argument
+    QRegExp re_rep1("replace(?::([bceprsw]*))?\\s+([\"'])((?:[^\\\\\\\\2]|\\\\.)*)\\2\\s*$");
+    // Else, it's just one or two (space separated) words
+    QRegExp re_rep2("replace(?::([bceprsw]*))?\\s+(\\S+)(.*)");
+#define unbackslash(s) p=0;\
+while ( (p = pattern.find( '\\' + delim, p )) > -1 )\
+{\
+  if ( !p || pattern[p-1] != '\\' )\
+    pattern.remove( p, 1 );\
+  p++;\
+}
+
+    if ( re_rep.search( cmd ) >= 0 )
+    {
+      flags = re_rep.cap(1);
+      pattern = re_rep.cap( 3 );
+      replacement = re_rep.cap( 4 );
+
+      int p(0);
+      // unbackslash backslashed delimiter strings
+      // in pattern ..
+      QString delim = re_rep.cap( 2 );
+      unbackslash(pattern);
+      // .. and in replacement
+      unbackslash(replacement);
+    }
+    else if ( re_rep1.search( cmd ) >= 0 )
+    {
+      flags = re_rep1.cap(1);
+      pattern = re_rep1.cap( 3 );
+
+      int p(0);
+      QString delim = re_rep1.cap( 2 );
+      unbackslash(pattern);
+    }
+    else if ( re_rep2.search( cmd ) >= 0 )
+    {
+      flags = re_rep2.cap( 1 );
+      pattern = re_rep2.cap( 2 );
+      replacement = re_rep2.cap( 3 ).stripWhiteSpace();
+    }
+    else
+    {
+      msg = i18n("Usage: replace[:[bceprsw]] PATTERN [REPLACEMENT]");
+      return false;
+    }
+    kdDebug()<<"replace '"<<pattern<<"' with '"<<replacement<<"'"<<endl;
+#undef unbackslash
+  }
+
+  long f = 0;
+  if ( flags.contains( 'b' ) ) f |= KFindDialog::FindBackwards;
+  if ( flags.contains( 'c' ) ) f |= KFindDialog::FromCursor;
+  if ( flags.contains( 'e' ) ) f |= KFindDialog::SelectedText;
+  if ( flags.contains( 'r' ) ) f |= KFindDialog::RegularExpression;
+  if ( flags.contains( 'p' ) ) f |= KReplaceDialog::PromptOnReplace;
+  if ( flags.contains( 's' ) ) f |= KFindDialog::CaseSensitive;
+  if ( flags.contains( 'w' ) ) f |= KFindDialog::WholeWordsOnly;
+
+  if ( cmd.startsWith( "find" ) )
+  {
+    ((KateView*)view)->find( pattern, f );
+    return true;
+  }
+  else if ( cmd.startsWith( "replace" ) )
+  {
+    f |= KReplaceDialog::BackReference; // mandatory here?
+    ((KateView*)view)->replace( pattern, replacement, f );
+    return true;
+  }
+
+  return false;
+}
+
+bool SearchCommand::help(class Kate::View *, const QString &cmd, QString &msg)
+{
+  if ( cmd == "find" )
+    msg = i18n("<p>Usage: <code>find[:bcersw] PATTERN</code></p>");
+
+  else if ( cmd == "ifind" )
+    msg = i18n("<p>Usage: <code>ifind:[:bcrs] PATTERN</code>"
+        "<br>ifind does incremental or 'as-you-type' search</p>");
+
+  else
+    msg = i18n("<p>Usage: <code>replace[:bceprsw] PATTERN REPLACEMENT</code></p>");
+
+  msg += i18n(
+      "<h4><caption>Options</h4><p>"
+      "<b>b</b> - Search backward"
+      "<br><b>c</b> - Search from cursor"
+      "<br><b>r</b> - Pattern is a regular expression"
+      "<br><b>s</b> - Case sensitive search"
+             );
+
+  if ( cmd == "find" )
+    msg += i18n(
+        "<br><b>e</b> - Search in selected text only"
+        "<br><b>w</b> - Search whole words only"
+               );
+
+  if ( cmd == "replace" )
+    msg += i18n(
+        "<br><b>p</b> - Prompt for replace</p>"
+        "<p>If REPLACEMENT is not present, an empty string is used.</p>"
+        "<p>If you want to have whitespace in your PATTERN, you need to "
+        "quote both PATTERN and REPLACEMENT with either single or double "
+        "quotes. To have the quote characters in the strings, prepend them "
+        "with a backslash.");
+
+  msg += "</p>";
+  return true;
+}
+
+QStringList SearchCommand::cmds()
+{
+  QStringList l;
+  l << "find" << "replace" << "ifind";
+  return l;
+}
+
+bool SearchCommand::wantsToProcessText( const QString &cmdname )
+{
+  return  cmdname == "ifind";
+}
+
+void SearchCommand::processText( Kate::View *view, const QString &cmd )
+{
+  static QRegExp re_ifind("ifind(?::([bcrs]*))?\\s(.*)");
+  if ( re_ifind.search( cmd ) > -1 )
+  {
+    QString flags = re_ifind.cap( 1 );
+    QString pattern = re_ifind.cap( 2 );
+
+
+    // if there is no setup, or the text length is 0, set up the properties
+    if ( ! m_ifindFlags || pattern.isEmpty() )
+      ifindInit( flags );
+    // if there is no fromCursor, add it if this is not the first character
+    else if ( ! ( m_ifindFlags & KFindDialog::FromCursor ) && ! pattern.isEmpty() )
+      m_ifindFlags |= KFindDialog::FromCursor;
+
+    // search..
+    if ( ! pattern.isEmpty() )
+    {
+      KateView *v = (KateView*)view;
+
+      // If it *looks like* we are continuing, place the cursor
+      // at the beginning of the selection, so that the search continues.
+      // ### check more carefully, like is  the cursor currently at the end
+      // of the selection.
+      if ( pattern.startsWith( v->getDoc()->selection() ) &&
+           v->getDoc()->selection().length() + 1 == pattern.length() )
+        v->setCursorPositionInternal( v->getDoc()->selStartLine(), v->getDoc()->selStartCol() );
+
+      v->find( pattern, m_ifindFlags, false );
+    }
+  }
+}
+
+void SearchCommand::ifindInit( const QString &flags )
+{
+  long f = 0;
+  if ( flags.contains( 'b' ) ) f |= KFindDialog::FindBackwards;
+  if ( flags.contains( 'c' ) ) f |= KFindDialog::FromCursor;
+  if ( flags.contains( 'r' ) ) f |= KFindDialog::RegularExpression;
+  if ( flags.contains( 's' ) ) f |= KFindDialog::CaseSensitive;
+  m_ifindFlags = f;
+}
+
+void SearchCommand::ifindClear()
+{
+  m_ifindFlags = 0;
+}
+//END SearchCommand
+
 // kate: space-indent on; indent-width 2; replace-tabs on;
