@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmpropmembers.h,v 1.2 2001/03/24 07:27:49 waba Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -26,7 +24,7 @@
 
 class QTextView;
 
-class KMPropMembers : public KMPropWidget
+class KDEPRINT_EXPORT KMPropMembers : public KMPropWidget
 {
 public:
 	KMPropMembers(QWidget *parent = 0, const char *name = 0);
