@@ -22,9 +22,11 @@
 
 #include <qwidget.h>
 
+#include <kdelibs_export.h>
+
 class KMPrinter;
 
-class KMWizardPage : public QWidget
+class KDEPRINT_EXPORT KMWizardPage : public QWidget
 {
 	Q_OBJECT
 public:
@@ -38,11 +40,13 @@ public:
 	virtual bool isValid(QString&);
 	virtual void initPrinter(KMPrinter*);
 	virtual void updatePrinter(KMPrinter*);
+	bool needsInitOnBack() 		{ return m_needsinitonback; }
 
 protected:
 	QString	m_title;
 	int	m_ID;
 	int	m_nextpage;
+	bool 	m_needsinitonback;
 };
 
 #endif
