@@ -53,7 +53,7 @@
 #include <kstandarddirs.h>
 #include <kdebug.h>
 #include <knuminput.h>
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
 #include <X11/Xlib.h> 
 		    // not needed any more...
 #endif
@@ -71,7 +71,7 @@ static int minimumListWidth( const QListBox *list )
   if( w == 0 ) { w = 40; }
   w += list->frameWidth() * 2;
   w += list->verticalScrollBar()->sizeHint().width();
-  return( w );
+  return w;
 }
 
 static int minimumListHeight( const QListBox *list, int numVisibleEntry )
@@ -81,7 +81,7 @@ static int minimumListHeight( const QListBox *list, int numVisibleEntry )
 
   if( w < 0 ) { w = 10; }
   if( numVisibleEntry <= 0 ) { numVisibleEntry = 4; }
-  return( w * numVisibleEntry + 2 * list->frameWidth() );
+  return ( w * numVisibleEntry + 2 * list->frameWidth() );
 }
 
 class KFontChooser::KFontChooserPrivate
@@ -112,7 +112,7 @@ KFontChooser::KFontChooser(QWidget *parent, const char *name,
   QWidget *page;
   QGridLayout *gridLayout;
   int row = 0;
-  if( makeFrame == true )
+  if( makeFrame )
   {
     page = new QGroupBox( i18n("Requested Font"), this );
     topLayout->addWidget(page);
@@ -205,7 +205,7 @@ KFontChooser::KFontChooser(QWidget *parent, const char *name,
   QWhatsThis::add(diff?(QWidget *) familyCheckbox:(QWidget *) familyLabel, fontFamilyWhatsThisText );
   connect(familyListBox, SIGNAL(highlighted(const QString &)),
 	  SLOT(family_chosen_slot(const QString &)));
-  if(fontList.count() != 0)
+  if(!fontList.isEmpty())
   {
     familyListBox->insertStringList(fontList);
   }
@@ -308,7 +308,7 @@ KFontChooser::KFontChooser(QWidget *parent, const char *name,
 	  SLOT(displaySample(const QFont &)));
 
   QVBoxLayout *vbox;
-  if( makeFrame == true )
+  if( makeFrame )
   {
     page = new QGroupBox( i18n("Actual Font"), this );
     topLayout->addWidget(page);
@@ -356,7 +356,7 @@ void KFontChooser::fillSizeList() {
     28, 32, 48, 64,
     0
   };
-  for(int i = 0; c[i] != 0; i++)
+  for(int i = 0; c[i]; ++i)
   {
     sizeListBox->insertItem(QString::number(c[i]));
   }
@@ -408,7 +408,7 @@ QButton::ToggleState KFontChooser::sizeIsRelative() const
 
 QSize KFontChooser::sizeHint( void ) const
 {
-  return( minimumSizeHint() );
+  return minimumSizeHint();
 }
 
 
@@ -478,7 +478,7 @@ void KFontChooser::family_chosen_slot(const QString& family)
         if(pos >=0) style = style.replace(pos,6,i18n("Regular"));
         pos = style.find("Oblique");
         if(pos >=0) style = style.replace(pos,7,i18n("Italic"));
-        if(styleListBox->findItem(style) ==0) {
+        if(!styleListBox->findItem(style)) {
             styleListBox->insertItem(i18n(style.utf8()));
             currentStyles.insert(i18n(style.utf8()), *it);
         }
@@ -712,7 +712,7 @@ void KFontChooser::fillFamilyListBox(bool onlyFixedFonts)
 
 void KFontChooser::showXLFDArea(bool show)
 {
-  if( show == true )
+  if( show )
   {
     xlfdEdit->parentWidget()->show();
   }
@@ -753,7 +753,7 @@ int KFontDialog::getFontDiff( QFont &theFont, int &diffFlags, bool onlyFixed,
     if( sizeIsRelativeState )
       *sizeIsRelativeState = dlg.chooser->sizeIsRelative();
   }
-  return( result );
+  return result;
 }
 
 int KFontDialog::getFont( QFont &theFont, bool onlyFixed,
@@ -771,7 +771,7 @@ int KFontDialog::getFont( QFont &theFont, bool onlyFixed,
     if( sizeIsRelativeState )
       *sizeIsRelativeState = dlg.chooser->sizeIsRelative();
   }
-  return( result );
+  return result;
 }
 
 
@@ -792,7 +792,7 @@ int KFontDialog::getFontAndText( QFont &theFont, QString &theString,
     if( sizeIsRelativeState )
       *sizeIsRelativeState = dlg.chooser->sizeIsRelative();
   }
-  return( result );
+  return result;
 }
 
 void KFontChooser::virtual_hook( int, void* )
