@@ -41,6 +41,8 @@ class QListViewItem;
 class KGuiItem;
 
 /**
+ * @short Easy to use widget with many layouts
+ *
  * Provides a number of ready to use layouts (faces). It is used
  * as an internal widget in KDialogBase, but can also used as a
  * widget of its own.
@@ -60,10 +62,9 @@ class KGuiItem;
  * the QObject::deleteLater() function on the page as the main event loop
  * may have optimized UI update events of the page by scheduling them for later.
  *
- * @short Easy to use widget with many layouts
  * @author Espen Sand (espen@kde.org)
  */
-class KJanusWidget : public QWidget
+class KDEUI_EXPORT KJanusWidget : public QWidget
 {
   Q_OBJECT
 
@@ -524,6 +525,11 @@ class KJanusWidget : public QWidget
     /*virtual*/ QWidget *pageWidget(int index) const;
 
   signals:
+    /**
+     * This signal is emitted whenever the current page changes.
+     * @param page the new page. 
+     * @since 3.4
+     */
     void aboutToShowPage(QWidget *page);
 
   public slots:
