@@ -102,6 +102,7 @@ public:
     int wordCount, errorCount;
     int checksRequested, checksDone;
     int disablePercentage;
+    int disableWordCount;
     bool completeRehighlightRequired;
     bool active, automatic, autoReady;
     bool globalConfig, spellReady;
@@ -291,9 +292,9 @@ KDictSpellingHighlighter::KDictSpellingHighlighter( QTextEdit *textEdit,
 
     KConfig *config = KGlobal::config();
     KConfigGroupSaver cs( config, "KSpell" );
-    config->reparseConfiguration();
     d->disablePercentage = config->readNumEntry( "KSpell_AsYouTypeDisablePercentage", 42 );
     d->disablePercentage = QMIN( d->disablePercentage, 101 );
+    d->disableWordCount = config->readNumEntry( "KSpell_AsYouTypeDisableWordCount", 100 );
 
     textEdit->installEventFilter( this );
     textEdit->viewport()->installEventFilter( this );
@@ -566,7 +567,7 @@ void KDictSpellingHighlighter::slotAutoDetection()
 
     if ( d->automatic ) {
 	// tme = Too many errors
-	bool tme = d->errorCount * 100 >= d->disablePercentage * d->wordCount;
+	bool tme = d->wordCount >= d->disableWordCount && d->errorCount * 100 >= d->disablePercentage * d->wordCount;
 	if ( d->active && tme )
 	    d->active = false;
 	else if ( !d->active && !tme )
