@@ -1,12 +1,12 @@
-
 /**
  * This file is part of the DOM implementation for KDE.
  *
  * Copyright (C) 1997 Martin Jones (mjones@kde.org)
  *           (C) 1997 Torben Weis (weis@kde.org)
  *           (C) 1998 Waldo Bastian (bastian@kde.org)
- *           (C) 1999 Lars Knoll (knoll@kde.org)
+ *           (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
+ *           (C) 2003 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,18 +22,15 @@
  * along with this library; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- *
- * $Id: html_tableimpl.cpp,v 1.126 2001/07/27 09:54:06 schlpbch Exp $
  */
-#include "html_tableimpl.h"
 
-#include "dom_exception.h"
-#include "html_documentimpl.h"
-#include "dom_node.h"
+#include "html/html_documentimpl.h"
+#include "html/html_tableimpl.h"
 
-using namespace DOM;
+#include "dom/dom_exception.h"
+#include "dom/dom_node.h"
 
-#include "htmlhashes.h"
+#include "misc/htmlhashes.h"
 #include "khtmlview.h"
 #include "khtml_part.h"
 
@@ -43,9 +40,12 @@ using namespace DOM;
 #include "css/csshelper.h"
 
 #include "rendering/render_table.h"
-using namespace khtml;
 
 #include <kdebug.h>
+#include <kglobal.h>
+
+using namespace khtml;
+using namespace DOM;
 
 HTMLTableElementImpl::HTMLTableElementImpl(DocumentPtr *doc)
   : HTMLElementImpl(doc)
@@ -55,19 +55,21 @@ HTMLTableElementImpl::HTMLTableElementImpl(DocumentPtr *doc)
     foot = 0;
     firstBody = 0;
 
+#if 0
     rules = None;
     frame = Void;
+#endif
+    padding = 1;
 
-    incremental = false;
     m_noBorder = true;
+    m_solid = false;
 
     // reset font color and sizes here, if we don't have strict parse mode.
     // this is 90% compatible to ie and mozilla, and the by way easiest solution...
     // only difference to 100% correct is that in strict mode <font> elements are propagated into tables.
-    if ( ownerDocument()->parseMode() != DocumentImpl::Strict ) {
+    if ( getDocument()->parseMode() < DocumentImpl::Transitional ) {
         addCSSProperty( CSS_PROP_FONT_SIZE, CSS_VAL_MEDIUM );
-        addCSSProperty( CSS_PROP_COLOR, ownerDocument()->textColor() );
-        addCSSProperty( CSS_PROP_FONT_FAMILY, "konq_default" );
+        addCSSProperty( CSS_PROP_COLOR, CSS_VAL__KHTML_TEXT );
     }
 }
 
@@ -75,19 +77,14 @@ HTMLTableElementImpl::~HTMLTableElementImpl()
 {
 }
 
-const DOMString HTMLTableElementImpl::nodeName() const
-{
-    return "TABLE";
-}
-
-ushort HTMLTableElementImpl::id() const
+NodeImpl::Id HTMLTableElementImpl::id() const
 {
     return ID_TABLE;
 }
 
 NodeImpl* HTMLTableElementImpl::setCaption( HTMLTableCaptionElementImpl *c )
 {
-    int exceptioncode;
+    int exceptioncode = 0;
     NodeImpl* r;
     if(tCaption) {
         replaceChild ( c, tCaption, exceptioncode );
@@ -101,7 +98,7 @@ NodeImpl* HTMLTableElementImpl::setCaption( HTMLTableCaptionElementImpl *c )
 
 NodeImpl* HTMLTableElementImpl::setTHead( HTMLTableSectionElementImpl *s )
 {
-    int exceptioncode;
+    int exceptioncode = 0;
     NodeImpl* r;
     if(head) {
         replaceChild( s, head, exceptioncode );
@@ -120,13 +117,12 @@ NodeImpl* HTMLTableElementImpl::setTHead( HTMLTableSectionElementImpl *s )
 
 NodeImpl* HTMLTableElementImpl::setTFoot( HTMLTableSectionElementImpl *s )
 {
-    int exceptioncode;
+    int exceptioncode = 0;
     NodeImpl* r;
     if(foot) {
         replaceChild ( s, foot, exceptioncode );
         r = s;
-    }
-    else if( firstBody )
+    } else if( firstBody )
         r = insertBefore( s, firstBody, exceptioncode );
     else
         r = appendChild( s, exceptioncode );
@@ -134,15 +130,30 @@ NodeImpl* HTMLTableElementImpl::setTFoot( HTMLTableSectionElementImpl *s )
     return r;
 }
 
+NodeImpl* HTMLTableElementImpl::setTBody( HTMLTableSectionElementImpl *s )
+{
+    int exceptioncode = 0;
+    NodeImpl* r;
+
+    if(firstBody) {
+        replaceChild ( s, firstBody, exceptioncode );
+        r = s;
+    } else
+        r = appendChild( s, exceptioncode );
+    firstBody = s;
+
+    return r;
+}
+
 HTMLElementImpl *HTMLTableElementImpl::createTHead(  )
 {
     if(!head)
     {
-        int exceptioncode;
-        head = new HTMLTableSectionElementImpl(docPtr(), ID_THEAD);
+        int exceptioncode = 0;
+        head = new HTMLTableSectionElementImpl(docPtr(), ID_THEAD, true /* implicit */);
         if(foot)
             insertBefore( head, foot, exceptioncode );
-        if(firstBody)
+        else if(firstBody)
             insertBefore( head, firstBody, exceptioncode);
         else
             appendChild(head, exceptioncode);
@@ -153,7 +164,7 @@ HTMLElementImpl *HTMLTableElementImpl::createTHead(  )
 void HTMLTableElementImpl::deleteTHead(  )
 {
     if(head) {
-        int exceptioncode;
+        int exceptioncode = 0;
         HTMLElementImpl::removeChild(head, exceptioncode);
     }
     head = 0;
@@ -163,8 +174,8 @@ HTMLElementImpl *HTMLTableElementImpl::createTFoot(  )
 {
     if(!foot)
     {
-        int exceptioncode;
-        foot = new HTMLTableSectionElementImpl(docPtr(), ID_TFOOT);
+        int exceptioncode = 0;
+        foot = new HTMLTableSectionElementImpl(docPtr(), ID_TFOOT, true /*implicit */);
         if(firstBody)
             insertBefore( foot, firstBody, exceptioncode );
         else
@@ -176,7 +187,7 @@ HTMLElementImpl *HTMLTableElementImpl::createTFoot(  )
 void HTMLTableElementImpl::deleteTFoot(  )
 {
     if(foot) {
-        int exceptioncode;
+        int exceptioncode = 0;
         HTMLElementImpl::removeChild(foot, exceptioncode);
     }
     foot = 0;
@@ -186,7 +197,7 @@ HTMLElementImpl *HTMLTableElementImpl::createCaption(  )
 {
     if(!tCaption)
     {
-        int exceptioncode;
+        int exceptioncode = 0;
         tCaption = new HTMLTableCaptionElementImpl(docPtr());
         insertBefore( tCaption, firstChild(), exceptioncode );
     }
@@ -196,21 +207,104 @@ HTMLElementImpl *HTMLTableElementImpl::createCaption(  )
 void HTMLTableElementImpl::deleteCaption(  )
 {
     if(tCaption) {
-        int exceptioncode;
+        int exceptioncode = 0;
         HTMLElementImpl::removeChild(tCaption, exceptioncode);
     }
     tCaption = 0;
 }
 
-HTMLElementImpl *HTMLTableElementImpl::insertRow( long /*index*/ )
+HTMLElementImpl *HTMLTableElementImpl::insertRow( long index, int &exceptioncode )
 {
-    // ###
-    return 0;
+    // The DOM requires that we create a tbody if the table is empty
+    // (cf DOM2TS HTMLTableElement31 test)
+    // (note: this is different from "if the table has no sections", since we can have
+    // <TABLE><TR>)
+    if(!firstBody && !head && !foot && !hasChildNodes())
+        setTBody( new HTMLTableSectionElementImpl(docPtr(), ID_TBODY, true /* implicit */) );
+
+    kdDebug(6030) << k_funcinfo << index << endl;
+    HTMLTableSectionElementImpl* section = 0L;
+    HTMLTableSectionElementImpl* lastSection = 0L;
+    NodeImpl *node = firstChild();
+    // The DOM requires that index=-1 means 'append after last'
+    bool append = (index == -1);
+    bool found = false;
+    for ( ; node && (index>=0 || append) ; node = node->nextSibling() )
+    {
+	// there could be 2 tfoot elements in the table. Only the first one is the "foot", that's why we have the more
+	// complicated if statement below.
+        if ( node != foot && (node->id() == ID_THEAD || node->id() == ID_TFOOT || node->id() == ID_TBODY) )
+        {
+            section = static_cast<HTMLTableSectionElementImpl *>(node);
+            lastSection = section;
+            //kdDebug(6030) << k_funcinfo << "section=" << section->tagName() << " rows:" << section->numRows() << endl;
+            if ( !append )
+            {
+                int rows = section->numRows();
+                if ( rows > index ) {
+		    found = true;
+                    break;
+                } else
+                    index -= rows;
+                //kdDebug(6030) << "       index is now " << index << endl;
+            }
+        }
+    }
+    // insertRow(numRows) appends to TFOOT. insertRow(-1) appends to TBODY, hence the !append.
+    if ( !found && !append )
+        section = static_cast<HTMLTableSectionElementImpl *>(foot);
+
+    // If index has decreased to 0, it means "insert before first row in current section"
+    // or "append after last row" (if there's no current section anymore)
+    if ( !section && ( index == 0 || append ) )
+    {
+        section = lastSection;
+        index = section ? section->numRows() : 0;
+    }
+    if ( section && (index >= 0 || append) ) {
+        //kdDebug(6030) << "Inserting row into section " << section << "(" << section->tagName().string() << ") at index " << index << endl;
+        return section->insertRow( index, exceptioncode );
+    } else {
+        // No more sections => index is too big
+        exceptioncode = DOMException::INDEX_SIZE_ERR;
+        return 0L;
+    }
 }
 
-void HTMLTableElementImpl::deleteRow( long /*index*/ )
+void HTMLTableElementImpl::deleteRow( long index, int &exceptioncode )
 {
-    // ###
+    HTMLTableSectionElementImpl* section = 0L;
+    NodeImpl *node = firstChild();
+    bool lastRow = index == -1;
+    HTMLTableSectionElementImpl* lastSection = 0L;
+    bool found = false;
+    for ( ; node ; node = node->nextSibling() )
+    {
+        if ( node != foot && (node->id() == ID_THEAD || node->id() == ID_TFOOT || node->id() == ID_TBODY) )
+        {
+            section = static_cast<HTMLTableSectionElementImpl *>(node);
+            lastSection = section;
+            int rows = section->numRows();
+            if ( !lastRow )
+            {
+                if ( rows > index ) {
+                    found = true;
+                    break;
+                } else
+                    index -= rows;
+            }
+        }
+        section = 0L;
+    }
+    if ( !found && foot )
+        section = static_cast<HTMLTableSectionElementImpl *>(foot);
+
+    if ( lastRow )
+        lastSection->deleteRow( -1, exceptioncode );
+    else if ( section && index >= 0 && index < section->numRows() )
+        section->deleteRow( index, exceptioncode );
+    else
+        exceptioncode = DOMException::INDEX_SIZE_ERR;
 }
 
 NodeImpl *HTMLTableElementImpl::addChild(NodeImpl *child)
@@ -219,47 +313,42 @@ NodeImpl *HTMLTableElementImpl::addChild(NodeImpl *child)
     kdDebug( 6030 ) << nodeName().string() << "(Table)::addChild( " << child->nodeName().string() << " )" << endl;
 #endif
 
-    switch(child->id())
-    {
-    case ID_CAPTION:
-        return setCaption(static_cast<HTMLTableCaptionElementImpl *>(child));
-        break;
-    case ID_COL:
-    case ID_COLGROUP:
-        {
-        // these have to come before the table definition!
-        if(head || foot || firstBody)
-            return 0;
-        HTMLElementImpl::addChild(child);
-        // ###
-        }
-        return child;
-    case ID_THEAD:
-        //      if(incremental && !columnPos[totalCols]);// calcColWidth();
-        return setTHead(static_cast<HTMLTableSectionElementImpl *>(child));
-        break;
-    case ID_TFOOT:
-        //if(incremental && !columnPos[totalCols]);// calcColWidth();
-        return setTFoot(static_cast<HTMLTableSectionElementImpl *>(child));
-        break;
-    case ID_TBODY:
-        //if(incremental && !columnPos[totalCols]);// calcColWidth();
-        if(!firstBody)
-            firstBody = static_cast<HTMLTableSectionElementImpl *>(child);
-        return HTMLElementImpl::addChild( child );
-        break;
+    int exceptioncode = 0;
+    NodeImpl *retval = appendChild( child, exceptioncode );
+    if ( retval ) {
+	switch(child->id()) {
+	case ID_CAPTION:
+	    if ( !tCaption )
+		tCaption = static_cast<HTMLTableCaptionElementImpl *>(child);
+	    break;
+	case ID_COL:
+	case ID_COLGROUP:
+	    break;
+	case ID_THEAD:
+	    if ( !head )
+		head = static_cast<HTMLTableSectionElementImpl *>(child);
+	    break;
+	case ID_TFOOT:
+	    if ( !foot )
+		foot = static_cast<HTMLTableSectionElementImpl *>(child);
+	    break;
+	case ID_TBODY:
+	    if ( !firstBody )
+		firstBody = static_cast<HTMLTableSectionElementImpl *>(child);
+	    break;
+	}
     }
-    return 0;
+    return retval;
 }
 
-void HTMLTableElementImpl::parseAttribute(AttrImpl *attr)
+void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
 {
     // ### to CSS!!
-    switch(attr->attrId)
+    switch(attr->id())
     {
     case ATTR_WIDTH:
         if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_WIDTH, attr->value());
+            addCSSLength( CSS_PROP_WIDTH, attr->value() );
         else
             removeCSSProperty(CSS_PROP_WIDTH);
         break;
@@ -284,7 +373,8 @@ void HTMLTableElementImpl::parseAttribute(AttrImpl *attr)
         border=1;
 #endif
         m_noBorder = !border;
-        addCSSLength(CSS_PROP_BORDER_WIDTH, DOMString( QString::number( border) ) );
+        DOMString v = QString::number( border );
+        addCSSLength(CSS_PROP_BORDER_WIDTH, v );
 #if 0
         // wanted by HTML4 specs
         if(!border)
@@ -296,26 +386,25 @@ void HTMLTableElementImpl::parseAttribute(AttrImpl *attr)
     }
     case ATTR_BGCOLOR:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_BACKGROUND_COLOR, attr->value());
+            addHTMLColor(CSS_PROP_BACKGROUND_COLOR, attr->value());
         else
             removeCSSProperty(CSS_PROP_BACKGROUND_COLOR);
         break;
     case ATTR_BORDERCOLOR:
         if(!attr->value().isEmpty()) {
-            addCSSProperty(CSS_PROP_BORDER_COLOR, attr->value());
+            addHTMLColor(CSS_PROP_BORDER_COLOR, attr->value());
             addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
+            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
+            m_solid = true;
         }
         break;
     case ATTR_BACKGROUND:
     {
         if (!attr->value().isEmpty()) {
-            HTMLDocumentImpl *doc = static_cast<HTMLDocumentImpl *>(ownerDocument());
             QString url = khtml::parseURL( attr->value() ).string();
-            if ( doc->view() )
-                url = doc->view()->part()->completeURL( url ).url();
+            url = getDocument()->completeURL( url );
             addCSSProperty(CSS_PROP_BACKGROUND_IMAGE, "url('"+url+"')" );
         }
         else
@@ -360,22 +449,19 @@ void HTMLTableElementImpl::parseAttribute(AttrImpl *attr)
         break;
    case ATTR_CELLSPACING:
         if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_BORDER_SPACING, attr->value());
+            addCSSLength(CSS_PROP_BORDER_SPACING, attr->value(), true);
         else
             removeCSSProperty(CSS_PROP_BORDER_SPACING);
         break;
     case ATTR_CELLPADDING:
-        if (!attr->value().isEmpty()) {
-            addCSSLength(CSS_PROP_PADDING_TOP, attr->value());
-            addCSSLength(CSS_PROP_PADDING_LEFT, attr->value());
-            addCSSLength(CSS_PROP_PADDING_BOTTOM, attr->value());
-            addCSSLength(CSS_PROP_PADDING_RIGHT, attr->value());
-        }
-        else {
-            removeCSSProperty(CSS_PROP_PADDING_TOP);
-            removeCSSProperty(CSS_PROP_PADDING_LEFT);
-            removeCSSProperty(CSS_PROP_PADDING_BOTTOM);
-            removeCSSProperty(CSS_PROP_PADDING_RIGHT);
+        if (!attr->value().isEmpty())
+	    padding = kMax( 0, attr->value().toInt() );
+        else
+	    padding = 1;
+        if (m_render && m_render->isTable()) {
+            static_cast<RenderTable *>(m_render)->setCellPadding(padding);
+	    if (!m_render->needsLayout())
+	        m_render->setNeedsLayout(true);
         }
         break;
     case ATTR_COLS:
@@ -385,23 +471,24 @@ void HTMLTableElementImpl::parseAttribute(AttrImpl *attr)
         int c;
         c = attr->val()->toInt();
         addColumns(c-totalCols);
-        break;
 #endif
+        break;
+
     }
     case ATTR_ALIGN:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_FLOAT, attr->value());
+            addCSSProperty(CSS_PROP_FLOAT, attr->value().lower());
         else
             removeCSSProperty(CSS_PROP_FLOAT);
         break;
     case ATTR_VALIGN:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value());
+            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value().lower());
         else
             removeCSSProperty(CSS_PROP_VERTICAL_ALIGN);
         break;
     case ATTR_NOSAVE:
-	break;	
+	break;
     default:
         HTMLElementImpl::parseAttribute(attr);
     }
@@ -409,29 +496,37 @@ void HTMLTableElementImpl::parseAttribute(AttrImpl *attr)
 
 void HTMLTableElementImpl::attach()
 {
+    if (!m_noBorder) {
+        int v = m_solid ? CSS_VAL_SOLID : CSS_VAL_OUTSET;
+        addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, v);
+        addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, v);
+        addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, v);
+        addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, v);
+    }
+
     HTMLElementImpl::attach();
+    if ( m_render && m_render->isTable() )
+	static_cast<RenderTable *>(m_render)->setCellPadding( padding );
 }
 
 // --------------------------------------------------------------------------
 
-void HTMLTablePartElementImpl::parseAttribute(AttrImpl *attr)
+void HTMLTablePartElementImpl::parseAttribute(AttributeImpl *attr)
 {
-    switch(attr->attrId)
+    switch(attr->id())
     {
     case ATTR_BGCOLOR:
         if (attr->val())
-            addCSSProperty(CSS_PROP_BACKGROUND_COLOR, attr->value() );
+            addHTMLColor(CSS_PROP_BACKGROUND_COLOR, attr->value() );
         else
             removeCSSProperty(CSS_PROP_BACKGROUND_COLOR);
         break;
     case ATTR_BACKGROUND:
     {
         if (attr->val()) {
-            HTMLDocumentImpl *doc = static_cast<HTMLDocumentImpl *>(ownerDocument());
             QString url = khtml::parseURL( attr->value() ).string();
-            if ( doc->view() )
-                url = doc->view()->part()->completeURL( url ).url();
-            addCSSProperty(CSS_PROP_BACKGROUND_IMAGE, "url('"+url+"')" );
+            url = getDocument()->completeURL( url );
+            addCSSProperty(CSS_PROP_BACKGROUND_IMAGE,  "url('"+url+"')" );
         }
         else
             removeCSSProperty(CSS_PROP_BACKGROUND_IMAGE);
@@ -440,54 +535,65 @@ void HTMLTablePartElementImpl::parseAttribute(AttrImpl *attr)
     case ATTR_BORDERCOLOR:
     {
         if(!attr->value().isEmpty()) {
-            addCSSProperty(CSS_PROP_BORDER_COLOR, attr->value());
+            addHTMLColor(CSS_PROP_BORDER_COLOR, attr->value());
             addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
+            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
         }
         break;
     }
+    case ATTR_ALIGN:
+    {
+        DOMString v = attr->value();
+        if ( strcasecmp( attr->value(), "middle" ) == 0 || strcasecmp( attr->value(), "center" ) == 0 )
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_CENTER);
+        else if (strcasecmp(attr->value(), "absmiddle") == 0)
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL_CENTER);
+        else if (strcasecmp(attr->value(), "left") == 0)
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_LEFT);
+        else if (strcasecmp(attr->value(), "right") == 0)
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_RIGHT);
+        else
+            addCSSProperty(CSS_PROP_TEXT_ALIGN, v);
+        break;
+    }
     case ATTR_VALIGN:
     {
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value());
+            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value().lower());
         else
             removeCSSProperty(CSS_PROP_VERTICAL_ALIGN);
         break;
     }
+    case ATTR_HEIGHT:
+        if (!attr->value().isEmpty())
+            addCSSLength(CSS_PROP_HEIGHT, attr->value());
+        else
+            removeCSSProperty(CSS_PROP_HEIGHT);
+        break;
     case ATTR_NOSAVE:
-	break;	
+	break;
     default:
         HTMLElementImpl::parseAttribute(attr);
     }
 }
 
-void HTMLTablePartElementImpl::attach()
-{
-    HTMLElementImpl::attach();
-}
-
 // -------------------------------------------------------------------------
 
 HTMLTableSectionElementImpl::HTMLTableSectionElementImpl(DocumentPtr *doc,
-                                                         ushort tagid)
+                                                         ushort tagid, bool implicit)
     : HTMLTablePartElementImpl(doc)
 {
     _id = tagid;
+    m_implicit = implicit;
 }
 
 HTMLTableSectionElementImpl::~HTMLTableSectionElementImpl()
 {
-    nrows = 0;
 }
 
-const DOMString HTMLTableSectionElementImpl::nodeName() const
-{
-    return getTagName(_id);
-}
-
-ushort HTMLTableSectionElementImpl::id() const
+NodeImpl::Id HTMLTableSectionElementImpl::id() const
 {
     return _id;
 }
@@ -495,84 +601,101 @@ ushort HTMLTableSectionElementImpl::id() const
 
 // these functions are rather slow, since we need to get the row at
 // the index... but they aren't used during usual HTML parsing anyway
-HTMLElementImpl *HTMLTableSectionElementImpl::insertRow( long index )
+HTMLElementImpl *HTMLTableSectionElementImpl::insertRow( long index, int& exceptioncode )
 {
-    nrows++;
-
-    HTMLTableRowElementImpl *r = new HTMLTableRowElementImpl(docPtr());
-
+    HTMLTableRowElementImpl *r = 0L;
     NodeListImpl *children = childNodes();
-    int exceptioncode;
-    if(!children || (int)children->length() <= index)
-        appendChild(r, exceptioncode);
-    else {
-        NodeImpl *n;
-        if(index < 1)
-            n = firstChild();
-        else
-            n = children->item(index);
-        insertBefore(r, n, exceptioncode );
+    int numRows = children ? (int)children->length() : 0;
+    //kdDebug(6030) << k_funcinfo << "index=" << index << " numRows=" << numRows << endl;
+    if ( index < -1 || index > numRows ) {
+        exceptioncode = DOMException::INDEX_SIZE_ERR; // per the DOM
     }
-    if(children) delete children;
+    else
+    {
+        r = new HTMLTableRowElementImpl(docPtr());
+        if ( numRows == index || index == -1 )
+            appendChild(r, exceptioncode);
+        else {
+            NodeImpl *n;
+            if(index < 1)
+                n = firstChild();
+            else
+                n = children->item(index);
+            insertBefore(r, n, exceptioncode );
+        }
+    }
+    delete children;
     return r;
 }
 
-void HTMLTableSectionElementImpl::deleteRow( long index )
+void HTMLTableSectionElementImpl::deleteRow( long index, int &exceptioncode )
 {
-    if(index < 0) return;
     NodeListImpl *children = childNodes();
-    if(children && (int)children->length() > index)
-    {
-        nrows--;
-        int exceptioncode;
+    int numRows = children ? (int)children->length() : 0;
+    if ( index == -1 ) index = numRows - 1;
+    if( index >= 0 && index < numRows )
         HTMLElementImpl::removeChild(children->item(index), exceptioncode);
+    else
+        exceptioncode = DOMException::INDEX_SIZE_ERR;
+    delete children;
+}
+
+
+int HTMLTableSectionElementImpl::numRows() const
+{
+    int rows = 0;
+    const NodeImpl *n = firstChild();
+    while (n) {
+        if (n->id() == ID_TR)
+            rows++;
+        n = n->nextSibling();
     }
-    if(children) delete children;
+
+    return rows;
 }
 
 // -------------------------------------------------------------------------
 
-HTMLTableRowElementImpl::HTMLTableRowElementImpl(DocumentPtr *doc)
-  : HTMLTablePartElementImpl(doc)
-{
-}
-
-HTMLTableRowElementImpl::~HTMLTableRowElementImpl()
-{
-}
-
-const DOMString HTMLTableRowElementImpl::nodeName() const
-{
-    return "TR";
-}
-
-ushort HTMLTableRowElementImpl::id() const
+NodeImpl::Id HTMLTableRowElementImpl::id() const
 {
     return ID_TR;
 }
 
 long HTMLTableRowElementImpl::rowIndex() const
 {
-    // some complex traversal stuff here to take into account that some rows may be in different sections
     int rIndex = 0;
-    const NodeImpl *n = this;
-    do {
-        while (!n->previousSibling() && !(n->isElementNode() && n->id() == ID_TABLE))
-            n = n->parentNode();
-        if (n->isElementNode() && n->id() == ID_TABLE)
-            n = 0;
-        if (n) {
-            n = n->previousSibling();
-            while (!(n->isElementNode() && n->id() == ID_TR) && n->lastChild())
-                n = n->lastChild();
-        }
 
-        if (n && n->isElementNode() && n->id() == ID_TR)
-            rIndex++;
+    NodeImpl *table = parentNode();
+    if ( !table )
+	return -1;
+    table = table->parentNode();
+    if ( !table || table->id() != ID_TABLE )
+	return -1;
+
+    HTMLTableSectionElementImpl *foot = static_cast<HTMLTableElementImpl *>(table)->tFoot();
+    NodeImpl *node = table->firstChild();
+    while ( node ) {
+        if ( node != foot && (node->id() == ID_THEAD || node->id() == ID_TFOOT || node->id() == ID_TBODY) ) {
+	    HTMLTableSectionElementImpl* section = static_cast<HTMLTableSectionElementImpl *>(node);
+	    const NodeImpl *row = section->firstChild();
+	    while ( row ) {
+		if ( row == this )
+		    return rIndex;
+		rIndex++;
+		row = row->nextSibling();
+	    }
+	}
+	node = node->nextSibling();
     }
-    while (n && n->isElementNode() && n->id() == ID_TR);
-
-    return rIndex;
+    const NodeImpl *row = foot->firstChild();
+    while ( row ) {
+	if ( row == this )
+	    return rIndex;
+	rIndex++;
+	row = row->nextSibling();
+    }
+    // should never happen
+    return -1;
 }
 
 long HTMLTableRowElementImpl::sectionRowIndex() const
@@ -589,35 +712,41 @@ long HTMLTableRowElementImpl::sectionRowIndex() const
     return rIndex;
 }
 
-HTMLElementImpl *HTMLTableRowElementImpl::insertCell( long index )
+HTMLElementImpl *HTMLTableRowElementImpl::insertCell( long index, int &exceptioncode )
 {
-    HTMLTableCellElementImpl *c = new HTMLTableCellElementImpl(docPtr(), ID_TD);
-
+    HTMLTableCellElementImpl *c = 0L;
     NodeListImpl *children = childNodes();
-    int exceptioncode;
-    if(!children || (int)children->length() <= index)
-        appendChild(c, exceptioncode);
-    else {
-        NodeImpl *n;
-        if(index < 1)
-            n = firstChild();
-        else
-            n = children->item(index);
-        insertBefore(c, n, exceptioncode);
+    int numCells = children ? children->length() : 0;
+    if ( index < -1 || index > numCells )
+        exceptioncode = DOMException::INDEX_SIZE_ERR; // per the DOM
+    else
+    {
+        c = new HTMLTableCellElementImpl(docPtr(), ID_TD);
+        if(numCells == index || index == -1)
+            appendChild(c, exceptioncode);
+        else {
+            NodeImpl *n;
+            if(index < 1)
+                n = firstChild();
+            else
+                n = children->item(index);
+            insertBefore(c, n, exceptioncode);
+        }
     }
-    if(children) delete children;
+    delete children;
     return c;
 }
 
-void HTMLTableRowElementImpl::deleteCell( long index )
+void HTMLTableRowElementImpl::deleteCell( long index, int &exceptioncode )
 {
-    if(index < 0) return;
     NodeListImpl *children = childNodes();
-    if(children && (int)children->length() > index) {
-        int exceptioncode;
+    int numCells = children ? children->length() : 0;
+    if ( index == -1 ) index = numCells-1;
+    if( index >= 0 && index < numCells )
         HTMLElementImpl::removeChild(children->item(index), exceptioncode);
-    }
-    if(children) delete children;
+    else
+        exceptioncode = DOMException::INDEX_SIZE_ERR;
+    delete children;
 }
 
 // -------------------------------------------------------------------------
@@ -628,29 +757,38 @@ HTMLTableCellElementImpl::HTMLTableCellElementImpl(DocumentPtr *doc, int tag)
   _col = -1;
   _row = -1;
   cSpan = rSpan = 1;
-  nWrap = false;
   _id = tag;
   rowHeight = 0;
+  m_solid = false;
 }
 
 HTMLTableCellElementImpl::~HTMLTableCellElementImpl()
 {
 }
 
-const DOMString HTMLTableCellElementImpl::nodeName() const
+long HTMLTableCellElementImpl::cellIndex() const
 {
-    return getTagName(_id);
+    return nodeIndex();
 }
 
-void HTMLTableCellElementImpl::parseAttribute(AttrImpl *attr)
+void HTMLTableCellElementImpl::parseAttribute(AttributeImpl *attr)
 {
-    switch(attr->attrId)
+    switch(attr->id())
     {
+    case ATTR_ALIGN:
+        if (attr->val()) {
+            if ( strcasecmp(attr->value(), "middle" ) == 0 ||
+                strcasecmp(attr->value(), "center" ) == 0 )
+                addCSSProperty( CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_CENTER );
+            else
+                addCSSProperty(CSS_PROP_TEXT_ALIGN, attr->value().lower());
+        }
+        else
+            removeCSSProperty(CSS_PROP_TEXT_ALIGN);
+        break;
     case ATTR_BORDER:
-        addCSSLength(CSS_PROP_BORDER_TOP_WIDTH, attr->value());
-        addCSSLength(CSS_PROP_BORDER_RIGHT_WIDTH, attr->value());
-        addCSSLength(CSS_PROP_BORDER_BOTTOM_WIDTH, attr->value());
-        addCSSLength(CSS_PROP_BORDER_LEFT_WIDTH, attr->value());
+        // euhm? not supported by other browsers as far as I can see (Dirk)
+        //addCSSLength(CSS_PROP_BORDER_WIDTH, attr->value());
         break;
     case ATTR_ROWSPAN:
         // ###
@@ -665,22 +803,19 @@ void HTMLTableCellElementImpl::parseAttribute(AttrImpl *attr)
         if(cSpan < 1 || cSpan > 1024) cSpan = 1;
         break;
     case ATTR_NOWRAP:
-        nWrap = (attr->val() != 0);
+        if (attr->val() != 0)
+	    addCSSProperty(CSS_PROP_WHITE_SPACE, CSS_VAL__KHTML_NOWRAP);
+        else
+	    removeCSSProperty(CSS_PROP_WHITE_SPACE);
         break;
     case ATTR_WIDTH:
         if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_WIDTH, attr->value());
+            addCSSLength( CSS_PROP_WIDTH, attr->value() );
         else
             removeCSSProperty(CSS_PROP_WIDTH);
         break;
-    case ATTR_HEIGHT:
-        if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_HEIGHT, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_HEIGHT);
-        break;
     case ATTR_NOSAVE:
-	break;	
+	break;
     default:
         HTMLTablePartElementImpl::parseAttribute(attr);
     }
@@ -688,145 +823,86 @@ void HTMLTableCellElementImpl::parseAttribute(AttrImpl *attr)
 
 void HTMLTableCellElementImpl::attach()
 {
-    HTMLElementImpl* p = static_cast<HTMLElementImpl*>(_parent);
+    HTMLElementImpl* p = static_cast<HTMLElementImpl*>(parentNode());
     while(p && p->id() != ID_TABLE)
         p = static_cast<HTMLElementImpl*>(p->parentNode());
 
     if(p) {
         HTMLTableElementImpl* table = static_cast<HTMLTableElementImpl*>(p);
-	//
-        if(table->m_noBorder && getAttribute(ATTR_BORDER).isNull()) {
-            addCSSProperty(CSS_PROP_BORDER_WIDTH, "0");
-	}
-        if(!table->getAttribute(ATTR_BORDERCOLOR).isNull()) {
-            addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
-	}
-    }
+        if (table->m_noBorder) {
+            removeCSSProperty(CSS_PROP_BORDER_WIDTH);
+        }
+        else {
+            addCSSProperty(CSS_PROP_BORDER_WIDTH, "1px");
+            int v = (table->m_solid || m_solid) ? CSS_VAL_SOLID : CSS_VAL_INSET;
+            addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, v);
+            addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, v);
+            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, v);
+            addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, v);
 
-    setStyle(ownerDocument()->styleSelector()->styleForElement(this));
-    khtml::RenderObject *r = _parent->renderer();
-    if(r)
-    {
-        m_render = khtml::RenderObject::createObject(this);
-        if(m_render && m_render->style()->display() == TABLE_CELL)
-        {
-            RenderTableCell *cell = static_cast<RenderTableCell *>(m_render);
-            cell->setRowSpan(rSpan);
-            cell->setColSpan(cSpan);
-            cell->setNoWrap(nWrap);
+            if (!m_solid)
+                addCSSProperty(CSS_PROP_BORDER_COLOR, CSS_VAL_INHERIT);
         }
-        if(m_render) r->addChild(m_render, nextRenderer());
     }
 
-    HTMLElementImpl::attach();
+    HTMLTablePartElementImpl::attach();
 }
 
-
-
 // -------------------------------------------------------------------------
 
 HTMLTableColElementImpl::HTMLTableColElementImpl(DocumentPtr *doc, ushort i)
-    : HTMLElementImpl(doc)
+    : HTMLTablePartElementImpl(doc)
 {
     _id = i;
-    _span = 1;
+    _span = (_id == ID_COLGROUP ? 0 : 1);
 }
 
-HTMLTableColElementImpl::~HTMLTableColElementImpl()
-{
-}
-
-const DOMString HTMLTableColElementImpl::nodeName() const
-{
-    return getTagName(_id);
-}
-
-ushort HTMLTableColElementImpl::id() const
+NodeImpl::Id HTMLTableColElementImpl::id() const
 {
     return _id;
 }
 
 
-NodeImpl *HTMLTableColElementImpl::addChild(NodeImpl *child)
+void HTMLTableColElementImpl::parseAttribute(AttributeImpl *attr)
 {
-#ifdef DEBUG_LAYOUT
-    kdDebug( 6030 ) << nodeName().string() << "(Table)::addChild( " << child->nodeName().string() << " )" << endl;
-#endif
-
-    switch(child->id())
-    {
-    case ID_COL:
-    {
-        // these have to come before the table definition!
-        HTMLElementImpl::addChild(child);
-        return child;
-    }
-    default:
-        return 0;
-        break;
-        // ####
-    }
-    return child;
-
-}
-
-void HTMLTableColElementImpl::parseAttribute(AttrImpl *attr)
-{
-    switch(attr->attrId)
+    switch(attr->id())
     {
     case ATTR_SPAN:
         _span = attr->val() ? attr->val()->toInt() : 1;
         break;
     case ATTR_WIDTH:
         if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_WIDTH, attr->value());
+            addCSSLength(CSS_PROP_WIDTH, attr->value(), false, true );
         else
             removeCSSProperty(CSS_PROP_WIDTH);
         break;
     case ATTR_VALIGN:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value());
+            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value().lower());
         else
             removeCSSProperty(CSS_PROP_VERTICAL_ALIGN);
         break;
     default:
-        HTMLElementImpl::parseAttribute(attr);
+        HTMLTablePartElementImpl::parseAttribute(attr);
     }
 
 }
 
 // -------------------------------------------------------------------------
 
-HTMLTableCaptionElementImpl::HTMLTableCaptionElementImpl(DocumentPtr *doc)
-  : HTMLTablePartElementImpl(doc)
-{
-}
-
-HTMLTableCaptionElementImpl::~HTMLTableCaptionElementImpl()
-{
-}
-
-const DOMString HTMLTableCaptionElementImpl::nodeName() const
-{
-    return "CAPTION";
-}
-
-ushort HTMLTableCaptionElementImpl::id() const
+NodeImpl::Id HTMLTableCaptionElementImpl::id() const
 {
     return ID_CAPTION;
 }
 
 
-void HTMLTableCaptionElementImpl::parseAttribute(AttrImpl *attr)
+void HTMLTableCaptionElementImpl::parseAttribute(AttributeImpl *attr)
 {
-    switch(attr->attrId)
+    switch(attr->id())
     {
     case ATTR_ALIGN:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_CAPTION_SIDE, attr->value());
+            addCSSProperty(CSS_PROP_CAPTION_SIDE, attr->value().lower());
         else
             removeCSSProperty(CSS_PROP_CAPTION_SIDE);
         break;
