@@ -4,7 +4,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2002 Apple Computer, Inc.
+ *           (C) 2002-2003 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -334,12 +334,18 @@ public:
     void setSelectedStylesheetSet(const DOMString&);
     void setPreferredStylesheetSet(const DOMString& s) { m_preferredStylesheetSet = s; }
 
+    void addStyleSheet(StyleSheetImpl *, int *exceptioncode = 0);
+    void removeStyleSheet(StyleSheetImpl *, int *exceptioncode = 0);
+
     QStringList availableStyleSheets() const { return m_availableSheets; }
 
     NodeImpl* hoverNode() const { return m_hoverNode; }
     void setHoverNode(NodeImpl *newHoverNode);
     NodeImpl *focusNode() const { return m_focusNode; }
     void setFocusNode(NodeImpl *newFocusNode);
+    // Updates for :target (CSS3 selector).
+    void setCSSTarget(NodeImpl* n);
+    NodeImpl* getCSSTarget() { return m_cssTarget; }
 
     bool isDocumentChanged()	{ return m_docChanged; }
     virtual void setDocumentChanged(bool = true);
@@ -484,6 +490,7 @@ protected:
     QColor m_textColor;
     NodeImpl *m_hoverNode;
     NodeImpl *m_focusNode;
+    NodeImpl *m_cssTarget;
 
     unsigned int m_domtree_version;
 
@@ -510,6 +517,7 @@ protected:
 
     unsigned short m_listenerTypes;
     StyleSheetListImpl* m_styleSheets;
+    StyleSheetListImpl *m_addedStyleSheets; // programmatically added style sheets
     LocalStyleRefs m_localStyleRefs; // references to inlined style elements
     QPtrList<RegisteredEventListener> m_windowEventListeners;
     QPtrList<NodeImpl> m_maintainsState;
