@@ -236,7 +236,7 @@ class KIconDialog::KIconDialogPrivate
  */
 
 KIconDialog::KIconDialog(QWidget *parent, const char *name)
-    : KDialogBase(parent, name, true, i18n("Select Icon"), Help|Ok|Cancel, Ok)
+    : KDialogBase(parent, name, true, i18n("Select Icon"), Ok|Cancel, Ok)
 {
     d = new KIconDialogPrivate;
     mpLoader = KGlobal::iconLoader();
@@ -245,7 +245,7 @@ KIconDialog::KIconDialog(QWidget *parent, const char *name)
 
 KIconDialog::KIconDialog(KIconLoader *loader, QWidget *parent,
 	const char *name)
-    : KDialogBase(parent, name, true, i18n("Select Icon"), Help|Ok|Cancel, Ok)
+    : KDialogBase(parent, name, true, i18n("Select Icon"), Ok|Cancel, Ok)
 {
     d = new KIconDialogPrivate;
     mpLoader = loader;
@@ -311,6 +311,7 @@ void KIconDialog::init()
 
     mpCanvas = new KIconCanvas(main);
     connect(mpCanvas, SIGNAL(executed(QIconViewItem *)), SLOT(slotAcceptIcons()));
+    connect(mpCanvas, SIGNAL(returnPressed(QIconViewItem *)), SLOT(slotAcceptIcons()));
     mpCanvas->setMinimumSize(400, 125);
     top->addWidget(mpCanvas);
     d->searchLine->setIconView(mpCanvas);
