@@ -18,6 +18,7 @@
 
 #include <unistd.h>
 
+#include <qapplication.h>
 #include <qcheckbox.h>
 #include <qdrawutil.h>
 #include <qfontmetrics.h>
@@ -25,6 +26,7 @@
 #include <qgrid.h>
 #include <qpainter.h>
 #include <qpopupmenu.h>
+#include <qstyle.h>
 #include <qvbox.h>
 #include <qwhatsthis.h>
 
@@ -180,6 +182,25 @@ void KURLBarItem::paint( QPainter *p )
     int w = width( box );
     static const int margin = KDialog::spacingHint();
 
+    // draw sunken selection
+    if ( isCurrent() || isSelected() ) {
+        int h = height( box );
+
+        QBrush brush = box->colorGroup().brush( QColorGroup::Highlight );
+        p->fillRect( 0, 0, w, h, brush );
+        QPen pen = p->pen();
+        QPen oldPen = pen;
+        pen.setColor( box->colorGroup().mid() );
+        p->setPen( pen );
+
+        p->drawPoint( 0, 0 );
+        p->drawPoint( 0, h - 1 );
+        p->drawPoint( w - 1, 0 );
+        p->drawPoint( w - 1, h - 1 );
+
+        p->setPen( oldPen );
+    }
+
     if ( m_parent->iconSize() < KIcon::SizeMedium ) {
         // small icon -> draw icon next to text
 
@@ -199,7 +220,16 @@ void KURLBarItem::paint( QPainter *p )
             yPos += margin;
             int stringWidth = box->width() - pm->width() - 2 - (margin * 2);
             QString visibleText = KStringHandler::rPixelSqueeze( text(), fm, stringWidth );
-            p->drawText( pm->width() + margin + 2, yPos, visibleText );
+            int xPos = pm->width() + margin + 2;
+
+            if ( isCurrent() || isSelected() ) {
+                p->setPen( box->colorGroup().highlight().dark(115) );
+                p->drawText( xPos + ( QApplication::reverseLayout() ? -1 : 1),
+                             yPos + 1, visibleText );
+                p->setPen( box->colorGroup().highlightedText() );
+            }
+
+            p->drawText( xPos, yPos, visibleText );
         }
         // end cut & paste (modulo pixmap centering)
     }
@@ -224,15 +254,16 @@ void KURLBarItem::paint( QPainter *p )
             int x = (w - fm.width( visibleText )) / 2;
             x = QMAX( x, margin );
 
+            if ( isCurrent() || isSelected() ) {
+                p->setPen( box->colorGroup().highlight().dark(115) );
+                p->drawText( x + ( QApplication::reverseLayout() ? -1 : 1),
+                             y + 1, visibleText );
+                p->setPen( box->colorGroup().highlightedText() );
+            }
+
             p->drawText( x, y, visibleText );
         }
     }
-
-    // draw sunken selection
-    if ( isCurrent() || isSelected() ) {
-        qDrawShadePanel( p, 1, 0, w -2, height(box),
-                         box->colorGroup(), true, 1, 0L );
-    }
 }
 
 QSize KURLBarItem::sizeHint() const
@@ -288,8 +319,10 @@ public:
     KURLBarPrivate()
     {
         currentURL.setPath( QDir::homeDirPath() );
+        defaultIconSize = 0;
     }
 
+    int defaultIconSize;
     KURL currentURL;
 };
 
@@ -326,15 +359,6 @@ KURLBarItem * KURLBar::insertItem(const KURL& url, const QString& description,
                                   bool applicationLocal,
                                   const QString& icon, KIcon::Group group )
 {
-// UGLY HACK, devices:/ and lan:/ don't work in dir selection dialog, better not show it
-if ((url.protocol() == "devices") || (url.protocol() == "lan"))
-{
-  QObject *obj = topLevelWidget();
-  if (obj && (qstrcmp(obj->className(), "KDirSelectDialog") == 0))
-     return 0;
-}
-// END UGLY HACK
-
     KURLBarItem *item = new KURLBarItem(this, url, description, icon, group);
     item->setApplicationLocal( applicationLocal );
     m_listBox->insertItem( item );
@@ -381,16 +405,7 @@ void KURLBar::setListBox( KURLBarListBox *view )
     }
 
     m_listBox->setSelectionMode( KListBox::Single );
-    QPalette pal = palette();
-    QColor gray = pal.color( QPalette::Normal, QColorGroup::Mid );
-    QColor selectedTextColor = pal.color( QPalette::Normal, QColorGroup::BrightText );
-    pal.setColor( QPalette::Normal,   QColorGroup::Base, gray );
-    pal.setColor( QPalette::Normal,   QColorGroup::HighlightedText, selectedTextColor );
-    pal.setColor( QPalette::Inactive, QColorGroup::Base, gray );
-    pal.setColor( QPalette::Inactive, QColorGroup::HighlightedText, selectedTextColor );
-
-    setPalette( pal );
-    m_listBox->viewport()->setBackgroundMode( PaletteMid );
+    paletteChange( palette() );
     m_listBox->setFocusPolicy( TabFocus );
 
     connect( m_listBox, SIGNAL( mouseButtonClicked( int, QListBoxItem *, const QPoint & ) ),
@@ -433,6 +448,22 @@ void KURLBar::resizeEvent( QResizeEvent *e )
     m_listBox->resize( width(), height() );
 }
 
+void KURLBar::paletteChange( const QPalette & )
+{
+    QPalette pal = palette();
+    QColor gray = pal.color( QPalette::Normal, QColorGroup::Background );
+    QColor selectedTextColor = pal.color( QPalette::Normal, QColorGroup::BrightText );
+    QColor foreground = pal.color( QPalette::Normal, QColorGroup::Foreground );
+    pal.setColor( QPalette::Normal,   QColorGroup::Base, gray );
+    pal.setColor( QPalette::Normal,   QColorGroup::HighlightedText, selectedTextColor );
+    pal.setColor( QPalette::Normal,   QColorGroup::Text, foreground );
+    pal.setColor( QPalette::Inactive, QColorGroup::Base, gray );
+    pal.setColor( QPalette::Inactive, QColorGroup::HighlightedText, selectedTextColor );
+    pal.setColor( QPalette::Inactive, QColorGroup::Text, foreground );
+
+    setPalette( pal );
+}
+
 QSize KURLBar::sizeHint() const
 {
     return m_listBox->sizeHint();
@@ -548,6 +579,7 @@ void KURLBar::readConfig( KConfig *appConfig, const QString& itemGroup )
 {
     m_isImmutable = appConfig->groupIsImmutable( itemGroup );
     KConfigGroupSaver cs( appConfig, itemGroup );
+    d->defaultIconSize = m_iconSize;
     m_iconSize = appConfig->readNumEntry( "Speedbar IconSize", m_iconSize );
 
     if ( m_useGlobal ) { // read global items
@@ -584,7 +616,10 @@ void KURLBar::readItem( int i, KConfig *config, bool applicationLocal )
 void KURLBar::writeConfig( KConfig *config, const QString& itemGroup )
 {
     KConfigGroupSaver cs1( config, itemGroup );
-    config->writeEntry( "Speedbar IconSize", m_iconSize );
+    if(!config->hasDefault("Speedbar IconSize") && m_iconSize == d->defaultIconSize )
+        config->revertToDefault("Speedbar IconSize");
+    else
+        config->writeEntry( "Speedbar IconSize", m_iconSize );
 
     if ( !m_isModified )
         return;
@@ -668,15 +703,9 @@ void KURLBar::slotDropped( QDropEvent *e )
 
         KURL::List::Iterator it = urls.begin();
         for ( ; it != urls.end(); ++it ) {
-            url = *it;
-            if ( KURLBarItemDialog::getInformation( m_useGlobal,
-                                                    url, description, icon,
-                                                    appLocal, m_iconSize,
-                                                    this ) ) {
-                (void) insertItem( url, description, appLocal, icon );
-                m_isModified = true;
-                updateGeometry();
-            }
+            (void) insertItem( *it, description, appLocal, icon );
+            m_isModified = true;
+            updateGeometry();
         }
     }
 }
@@ -799,6 +828,13 @@ KURLBarListBox::~KURLBarListBox()
     delete m_toolTip;
 }
 
+void KURLBarListBox::paintEvent( QPaintEvent* )
+{
+    QPainter p(this);
+    p.setPen( colorGroup().mid() );
+    p.drawRect( 0, 0, width(), height() );
+}
+
 QDragObject * KURLBarListBox::dragObject()
 {
     KURL::List urls;
