@@ -55,7 +55,7 @@ struct ListProgressColumnConfig
 * List view in the UIServer.
 * @internal
 */
-class ListProgress : public KListView {
+class KIO_EXPORT ListProgress : public KListView {
 
   Q_OBJECT
 
@@ -103,7 +103,7 @@ protected:
 * One item in the ListProgress
 * @internal
 */
-class ProgressItem : public QObject, public QListViewItem {
+class KIO_EXPORT ProgressItem : public QObject, public QListViewItem {
 
   Q_OBJECT
 
@@ -214,7 +214,7 @@ class UIServerSystemTray;
  *
  * @internal
  */
-class UIServer : public KMainWindow, public DCOPObject {
+class KIO_EXPORT UIServer : public KMainWindow, public DCOPObject {
 
   K_DCOP
   Q_OBJECT
@@ -349,11 +349,21 @@ k_dcop:
   /**
    * Show a SSL Information Dialog
    */
+  void showSSLInfoDialog(const QString &url, const KIO::MetaData &data, int mainwindow);
+
+  /**
+   * @deprecated
+   */
   void showSSLInfoDialog(const QString &url, const KIO::MetaData &data);
 
   /*
    * Show an SSL Certificate Selection Dialog
    */
+  KSSLCertDlgRet showSSLCertDialog(const QString& host, const QStringList& certList, int mainwindow);
+
+  /*
+   * @deprecated
+   */
   KSSLCertDlgRet showSSLCertDialog(const QString& host, const QStringList& certList);
 
 public slots:
