@@ -23,9 +23,6 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#ifndef HAVE_SYS_TIMEB_H
-#define HAVE_SYS_TIMEB_H 0
-#endif
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -37,20 +34,26 @@
 #  include <time.h>
 # endif
 #endif
-#if HAVE_SYS_TIMEB_H
+#ifdef HAVE_SYS_TIMEB_H
 #include <sys/timeb.h>
 #endif
 
+#include <errno.h>
+
 #ifdef HAVE_SYS_PARAM_H
 #  include <sys/param.h>
 #endif // HAVE_SYS_PARAM_H
 
 #include <math.h>
 #include <string.h>
+#ifdef HAVE_STRINGS_H
+#  include <strings.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <locale.h>
 #include <ctype.h>
+#include <assert.h>
 
 #include "date_object.h"
 #include "error_object.h"
@@ -58,10 +61,202 @@
 
 #include "date_object.lut.h"
 
-const time_t invalidDate = -1;
-
 using namespace KJS;
 
+// come constants
+const time_t invalidDate = LONG_MIN;
+const double hoursPerDay = 24;
+const double minutesPerHour = 60;
+const double secondsPerMinute = 60;
+const double msPerSecond = 1000;
+const double msPerMinute = msPerSecond * secondsPerMinute;
+const double msPerHour = msPerMinute * minutesPerHour;
+const double msPerDay = msPerHour * hoursPerDay;
+static const char * const weekdayName[7] = { "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun" };
+static const char * const monthName[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
+
+static UString formatDate(struct tm &tm)
+{
+    char buffer[100];
+    snprintf(buffer, sizeof(buffer), "%s %s %02d %04d",
+            weekdayName[(tm.tm_wday + 6) % 7],
+            monthName[tm.tm_mon], tm.tm_mday, tm.tm_year + 1900);
+    return buffer;
+}
+
+static UString formatDateUTCVariant(struct tm &tm)
+{
+    char buffer[100];
+    snprintf(buffer, sizeof(buffer), "%s, %02d %s %04d",
+            weekdayName[(tm.tm_wday + 6) % 7],
+            tm.tm_mday, monthName[tm.tm_mon], tm.tm_year + 1900);
+    return buffer;
+}
+
+static UString formatTime(struct tm &tm)
+{
+    char buffer[100];
+    if (tm.tm_gmtoff == 0) {
+        snprintf(buffer, sizeof(buffer), "%02d:%02d:%02d GMT", tm.tm_hour, tm.tm_min, tm.tm_sec);
+    } else {
+        int offset = tm.tm_gmtoff;
+        if (offset < 0) {
+            offset = -offset;
+        }
+        snprintf(buffer, sizeof(buffer), "%02d:%02d:%02d GMT%c%02d%02d",
+                tm.tm_hour, tm.tm_min, tm.tm_sec,
+                tm.tm_gmtoff < 0 ? '-' : '+', offset / (60*60), (offset / 60) % 60);
+    }
+    return UString(buffer);
+}
+
+static int day(double t)
+{
+  return int(floor(t / msPerDay));
+}
+
+static double dayFromYear(int year)
+{
+  return 365.0 * (year - 1970)
+    + floor((year - 1969) / 4.0)
+    - floor((year - 1901) / 100.0)
+    + floor((year - 1601) / 400.0);
+}
+
+// depending on whether it's a leap year or not
+static int daysInYear(int year)
+{
+  if (year % 4 != 0)
+    return 365;
+  else if (year % 400 == 0)
+    return 366;
+  else if (year % 100 == 0)
+    return 365;
+  else
+    return 366;
+}
+
+// time value of the start of a year
+double timeFromYear(int year)
+{
+  return msPerDay * dayFromYear(year);
+}
+
+// year determined by time value
+int yearFromTime(double t)
+{
+  // ### there must be an easier way
+  // initial guess
+  int y = 1970 + int(t / (365.25 * msPerDay));
+  // adjustment
+  if (timeFromYear(y) > t) {
+    do {
+      --y;
+    } while (timeFromYear(y) > t);
+  } else {
+    while (timeFromYear(y + 1) < t)
+      ++y;
+  }
+
+  return y;
+}
+
+// 0: Sunday, 1: Monday, etc.
+int weekDay(double t)
+{
+  int wd = (day(t) + 4) % 7;
+  if (wd < 0)
+    wd += 7;
+  return wd;
+}
+
+static double timeZoneOffset(const struct tm *t)
+{
+#if defined BSD || defined(__linux__) || defined(__APPLE__)
+  return -(t->tm_gmtoff / 60);
+#else
+#  if defined(__BORLANDC__)
+// FIXME consider non one-hour DST change
+#error please add daylight savings offset here!
+  return _timezone / 60 - (t->tm_isdst > 0 ? 60 : 0);
+#  else
+  return timezone / 60 - (t->tm_isdst > 0 ? 60 : 0 );
+#  endif
+#endif
+}
+
+// Converts a list of arguments sent to a Date member function into milliseconds, updating
+// ms (representing milliseconds) and t (representing the rest of the date structure) appropriately.
+//
+// Format of member function: f([hour,] [min,] [sec,] [ms])
+static void fillStructuresUsingTimeArgs(ExecState *exec, const List &args, int maxArgs, double *ms, struct tm *t)
+{
+    double milliseconds = 0;
+    int idx = 0;
+    int numArgs = args.size();
+    
+    // JS allows extra trailing arguments -- ignore them
+    if (numArgs > maxArgs)
+        numArgs = maxArgs;
+
+    // hours
+    if (maxArgs >= 4 && idx < numArgs) {
+        t->tm_hour = 0;
+        milliseconds += args[idx++].toInt32(exec) * msPerHour;
+    }
+
+    // minutes
+    if (maxArgs >= 3 && idx < numArgs) {
+        t->tm_min = 0;
+        milliseconds += args[idx++].toInt32(exec) * msPerMinute;
+    }
+    
+    // seconds
+    if (maxArgs >= 2 && idx < numArgs) {
+        t->tm_sec = 0;
+        milliseconds += args[idx++].toInt32(exec) * msPerSecond;
+    }
+    
+    // milliseconds
+    if (idx < numArgs) {
+        milliseconds += roundValue(exec, args[idx]);
+    } else {
+        milliseconds += *ms;
+    }
+    
+    *ms = milliseconds;
+}
+
+// Converts a list of arguments sent to a Date member function into years, months, and milliseconds, updating
+// ms (representing milliseconds) and t (representing the rest of the date structure) appropriately.
+//
+// Format of member function: f([years,] [months,] [days])
+static void fillStructuresUsingDateArgs(ExecState *exec, const List &args, int maxArgs, double *ms, struct tm *t)
+{
+  int idx = 0;
+  int numArgs = args.size();
+  
+  // JS allows extra trailing arguments -- ignore them
+  if (numArgs > maxArgs)
+    numArgs = maxArgs;
+  
+  // years
+  if (maxArgs >= 3 && idx < numArgs) {
+    t->tm_year = args[idx++].toInt32(exec) - 1900;
+  }
+  
+  // months
+  if (maxArgs >= 2 && idx < numArgs) {
+    t->tm_mon = args[idx++].toInt32(exec);
+  }
+  
+  // days
+  if (idx < numArgs) {
+    t->tm_mday = 0;
+    *ms += args[idx].toInt32(exec) * msPerDay;
+  }
+}
+
 // ------------------------------ DateInstanceImp ------------------------------
 
 const ClassInfo DateInstanceImp::info = {"Date", 0, 0, 0};
@@ -208,63 +403,63 @@ Value DateProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args)
       return Number(NaN);
     }
   }
-  time_t tv = (time_t) floor(milli / 1000.0);
-  int ms = int(milli - tv * 1000.0);
 
-  // As long as we're using time_t we need to 'truncate' to avoid 'wrapping'.
-  // Real long term solutions include: writing our own 64-bit-based date/time class,
-  // using wxWindow's datetime.cpp (in wxBase), using QDateTime... or shifting
-  // to a time_t range by substracting a big enough number of years....
-  if (sizeof(time_t) == 4)
-  {
-    // If time_t is signed, the bigger it can be is 2^31-1
-    if ( (time_t)-1 < 0 ) {
-      if ( floor(milli / 1000.0) > ((double)((uint)1<<31)-1) ) {
-#ifdef KJS_VERBOSE
-        fprintf(stderr, "date above time_t limit. Year seems to be %d\n", (int)(milli/(1000.0*365.25*86400)+1970));
-#endif
-        tv = ((uint)1<<31)-1;
-        ms = 0;
-      }
-    }
-    else
-      // time_t is unsigned, the bigger it can be is 2^32-1, aka (uint)-1
-      if ( floor(milli / 1000.0) > ((double)(uint)-1) ) {
-#ifdef KJS_VERBOSE
-        fprintf(stderr, "date above time_t limit. Year seems to be %d\n", (int)(milli/(1000.0*365.25*86400)+1970));
-#endif
-        tv = (uint)-1;
-        ms = 0;
-      }
+  // check whether time value is outside time_t's usual range
+  // make the necessary transformations if necessary
+  int realYearOffset = 0;
+  double milliOffset = 0.0;
+  if (milli < 0 || milli >= timeFromYear(2038)) {
+    // ### ugly and probably not very precise
+    int realYear = yearFromTime(milli);
+    int base = daysInYear(realYear) == 365 ? 2001 : 2000;
+    milliOffset = timeFromYear(base) - timeFromYear(realYear);
+    milli += milliOffset;
+    realYearOffset = realYear - base;
   }
 
+  time_t tv = (time_t) floor(milli / 1000.0);
+  double ms = milli - tv * 1000.0;
+
   struct tm *t;
-  if (utc)
+  if ( (id == DateProtoFuncImp::ToGMTString) ||
+       (id == DateProtoFuncImp::ToUTCString) )
+    t = gmtime(&tv);
+  else if (id == DateProtoFuncImp::ToString)
+    t = localtime(&tv);
+  else if (utc)
     t = gmtime(&tv);
   else
     t = localtime(&tv);
 
+  // we had an out of range year. use that one (plus/minus offset
+  // found by calculating tm_year) and fix the week day calculation
+  if (realYearOffset != 0) {
+    t->tm_year += realYearOffset;
+    milli -= milliOffset;
+    // our own weekday calculation. beware of need for local time.
+    double m = milli;
+    if (!utc)
+      m -= timeZoneOffset(t) * msPerMinute;
+    t->tm_wday = weekDay(m);
+  }
+
   // trick gcc. We don't want the Y2K warnings.
   const char xFormat[] = "%x";
   const char cFormat[] = "%c";
 
   switch (id) {
   case ToString:
+    result = String(formatDate(*t) + " " + formatTime(*t));
+    break;
   case ToDateString:
+    result = String(formatDate(*t));
+    break;
   case ToTimeString:
+    result = String(formatTime(*t));
+    break;
   case ToGMTString:
   case ToUTCString:
-    setlocale(LC_TIME,"C");
-    if (id == DateProtoFuncImp::ToDateString) {
-      strftime(timebuffer, bufsize, xFormat, t);
-    } else if (id == DateProtoFuncImp::ToTimeString) {
-      strftime(timebuffer, bufsize, "%X",t);
-    } else { // ToString, toGMTString & toUTCString
-      t = (id == ToString ? localtime(&tv) : gmtime(&tv));
-      strftime(timebuffer, bufsize, "%a, %d %b %Y %H:%M:%S %z", t);
-    }
-    setlocale(LC_TIME,oldlocale.c_str());
-    result = String(timebuffer);
+    result = String(formatDateUTCVariant(*t) + " " + formatTime(*t));
     break;
   case ToLocaleString:
     strftime(timebuffer, bufsize, cFormat, t);
@@ -316,16 +511,7 @@ Value DateProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args)
     result = Number(ms);
     break;
   case GetTimezoneOffset:
-#if defined BSD || defined(__APPLE__)
-    result = Number(-(t->tm_gmtoff / 60) + (t->tm_isdst > 0 ? 60 : 0));
-#else
-#  if defined(__BORLANDC__)
-#error please add daylight savings offset here!
-    result = Number(_timezone / 60 - (t->tm_isdst > 0 ? 60 : 0));
-#  else
-    result = Number((timezone / 60 - (t->tm_isdst > 0 ? 60 : 0 )));
-#  endif
-#endif
+    result = Number(timeZoneOffset(t));
     break;
   case SetTime:
     milli = roundValue(exec,args[0]);
@@ -333,53 +519,44 @@ Value DateProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args)
     thisObj.setInternalValue(result);
     break;
   case SetMilliSeconds:
-    ms = args[0].toInt32(exec);
+    fillStructuresUsingTimeArgs(exec, args, 1, &ms, t);
     break;
   case SetSeconds:
-    t->tm_sec = args[0].toInt32(exec);
-    if (args.size() >= 2)
-      ms = args[1].toInt32(exec);
+    fillStructuresUsingTimeArgs(exec, args, 2, &ms, t);
     break;
   case SetMinutes:
-    t->tm_min = args[0].toInt32(exec);
-    if (args.size() >= 2)
-      t->tm_sec = args[1].toInt32(exec);
-    if (args.size() >= 3)
-      ms = args[2].toInt32(exec);
+    fillStructuresUsingTimeArgs(exec, args, 3, &ms, t);
     break;
   case SetHours:
-    t->tm_hour = args[0].toInt32(exec);
-    if (args.size() >= 2)
-      t->tm_min = args[1].toInt32(exec);
-    if (args.size() >= 3)
-      t->tm_sec = args[2].toInt32(exec);
-    if (args.size() >= 4)
-      ms = args[3].toInt32(exec);
+    fillStructuresUsingTimeArgs(exec, args, 4, &ms, t);
     break;
   case SetDate:
-    t->tm_mday = args[0].toInt32(exec);
+    fillStructuresUsingDateArgs(exec, args, 1, &ms, t);
     break;
   case SetMonth:
-    t->tm_mon = args[0].toInt32(exec);
-    if (args.size() >= 2)
-      t->tm_mday = args[1].toInt32(exec);
+    fillStructuresUsingDateArgs(exec, args, 2, &ms, t);    
     break;
   case SetFullYear:
-    t->tm_year = args[0].toInt32(exec) - 1900;
-    if (args.size() >= 2)
-      t->tm_mon = args[1].toInt32(exec);
-    if (args.size() >= 3)
-      t->tm_mday = args[2].toInt32(exec);
+    fillStructuresUsingDateArgs(exec, args, 3, &ms, t);
     break;
   case SetYear:
-    t->tm_year = args[0].toInt32(exec) >= 1900 ? args[0].toInt32(exec) - 1900 : args[0].toInt32(exec);
+    int y = args[0].toInt32(exec);
+    if (y < 1900) {
+      if (y >= 0 && y <= 99) {
+        t->tm_year = y;
+      } else {
+        fillStructuresUsingDateArgs(exec, args, 3, &ms, t);
+      }
+    } else {
+      t->tm_year = y - 1900;
+    }
     break;
   }
 
   if (id == SetYear || id == SetMilliSeconds || id == SetSeconds ||
       id == SetMinutes || id == SetHours || id == SetDate ||
       id == SetMonth || id == SetFullYear ) {
-    result = Number(mktime(t) * 1000.0 + ms);
+    result = Number(makeTime(t, ms, utc));
     thisObj.setInternalValue(result);
   }
 
@@ -422,10 +599,10 @@ Object DateObjectImp::construct(ExecState *exec, const List &args)
 #ifdef KJS_VERBOSE
   fprintf(stderr,"DateObjectImp::construct - %d args\n", numArgs);
 #endif
-  Value value;
+  double value;
 
   if (numArgs == 0) { // new Date() ECMA 15.9.3.3
-#if HAVE_SYS_TIMEB_H
+#ifdef HAVE_SYS_TIMEB_H
 #  if defined(__BORLANDC__)
     struct timeb timebuffer;
     ftime(&timebuffer);
@@ -439,33 +616,41 @@ Object DateObjectImp::construct(ExecState *exec, const List &args)
     gettimeofday(&tv, 0L);
     double utc = floor((double)tv.tv_sec * 1000.0 + (double)tv.tv_usec / 1000.0);
 #endif
-    value = Number(utc);
+    value = utc;
   } else if (numArgs == 1) {
-    UString s = args[0].toString(exec);
-    double d = s.toDouble();
-    if (isNaN(d))
-      value = parseDate(s);
+    Value prim = args[0].toPrimitive(exec);
+    if (prim.isA(StringType))
+      value = parseDate(prim.toString(exec));
     else
-      value = Number(d);
+      value = prim.toNumber(exec);
   } else {
-    struct tm t;
-    memset(&t, 0, sizeof(t));
-    int year = args[0].toInt32(exec);
-    // TODO: check for NaN
-    t.tm_year = (year >= 0 && year <= 99) ? year : year - 1900;
-    t.tm_mon = args[1].toInt32(exec);
-    t.tm_mday = (numArgs >= 3) ? args[2].toInt32(exec) : 1;
-    t.tm_hour = (numArgs >= 4) ? args[3].toInt32(exec) : 0;
-    t.tm_min = (numArgs >= 5) ? args[4].toInt32(exec) : 0;
-    t.tm_sec = (numArgs >= 6) ? args[5].toInt32(exec) : 0;
-    t.tm_isdst = -1;
-    int ms = (numArgs >= 7) ? args[6].toInt32(exec) : 0;
-    value = Number(mktime(&t) * 1000.0 + ms);
+    if (isNaN(args[0].toNumber(exec))
+        || isNaN(args[1].toNumber(exec))
+        || (numArgs >= 3 && isNaN(args[2].toNumber(exec)))
+        || (numArgs >= 4 && isNaN(args[3].toNumber(exec)))
+        || (numArgs >= 5 && isNaN(args[4].toNumber(exec)))
+        || (numArgs >= 6 && isNaN(args[5].toNumber(exec)))
+        || (numArgs >= 7 && isNaN(args[6].toNumber(exec)))) {
+      value = NaN;
+    } else {
+      struct tm t;
+      memset(&t, 0, sizeof(t));
+      int year = args[0].toInt32(exec);
+      t.tm_year = (year >= 0 && year <= 99) ? year : year - 1900;
+      t.tm_mon = args[1].toInt32(exec);
+      t.tm_mday = (numArgs >= 3) ? args[2].toInt32(exec) : 1;
+      t.tm_hour = (numArgs >= 4) ? args[3].toInt32(exec) : 0;
+      t.tm_min = (numArgs >= 5) ? args[4].toInt32(exec) : 0;
+      t.tm_sec = (numArgs >= 6) ? args[5].toInt32(exec) : 0;
+      t.tm_isdst = -1;
+      int ms = (numArgs >= 7) ? args[6].toInt32(exec) : 0;
+      value = makeTime(&t, ms, false);
+    }
   }
 
   Object proto = exec->interpreter()->builtinDatePrototype();
   Object ret(new DateInstanceImp(proto.imp()));
-  ret.setInternalValue(timeClip(value));
+  ret.setInternalValue(Number(timeClip(value)));
   return ret;
 }
 
@@ -481,10 +666,9 @@ Value DateObjectImp::call(ExecState* /*exec*/, Object &/*thisObj*/, const List &
   fprintf(stderr,"DateObjectImp::call - current time\n");
 #endif
   time_t t = time(0L);
-  UString s(ctime(&t));
-
-  // return formatted string minus trailing \n
-  return String(s.substr(0, s.size() - 1));
+  // FIXME: not threadsafe
+  struct tm *tm = localtime(&t);
+  return String(formatDate(*tm) + " " + formatTime(*tm));
 }
 
 // ------------------------------ DateObjectFuncImp ----------------------------
@@ -506,13 +690,22 @@ bool DateObjectFuncImp::implementsCall() const
 Value DateObjectFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args)
 {
   if (id == Parse) {
-    return parseDate(args[0].toString(exec));
+    return Number(parseDate(args[0].toString(exec)));
   } else { // UTC
+    int n = args.size();
+    if (isNaN(args[0].toNumber(exec))
+        || isNaN(args[1].toNumber(exec))
+        || (n >= 3 && isNaN(args[2].toNumber(exec)))
+        || (n >= 4 && isNaN(args[3].toNumber(exec)))
+        || (n >= 5 && isNaN(args[4].toNumber(exec)))
+        || (n >= 6 && isNaN(args[5].toNumber(exec)))
+        || (n >= 7 && isNaN(args[6].toNumber(exec)))) {
+      return Number(NaN);
+    }
+
     struct tm t;
     memset(&t, 0, sizeof(t));
-    int n = args.size();
     int year = args[0].toInt32(exec);
-    // TODO: check for NaN
     t.tm_year = (year >= 0 && year <= 99) ? year : year - 1900;
     t.tm_mon = args[1].toInt32(exec);
     t.tm_mday = (n >= 3) ? args[2].toInt32(exec) : 1;
@@ -520,37 +713,21 @@ Value DateObjectFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &
     t.tm_min = (n >= 5) ? args[4].toInt32(exec) : 0;
     t.tm_sec = (n >= 6) ? args[5].toInt32(exec) : 0;
     int ms = (n >= 7) ? args[6].toInt32(exec) : 0;
-    return Number(mktime(&t) * 1000.0 + ms);
+    return Number(makeTime(&t, ms, true));
   }
 }
 
 // -----------------------------------------------------------------------------
 
 
-Value KJS::parseDate(const UString &u)
+double KJS::parseDate(const UString &u)
 {
 #ifdef KJS_VERBOSE
   fprintf(stderr,"KJS::parseDate %s\n",u.ascii());
 #endif
   double /*time_t*/ seconds = KRFCDate_parseDate( u );
-#ifdef KJS_VERBOSE
-  fprintf(stderr,"KRFCDate_parseDate returned seconds=%g\n",seconds);
-  bool withinLimits = true;
-  if ( sizeof(time_t) == 4 )
-  {
-    int limit = ((time_t)-1 < 0) ? 2038 : 2115;
-    if ( seconds > (limit-1970) * 365.25 * 86400 ) {
-      fprintf(stderr, "date above time_t limit. Year seems to be %d\n", (int)(seconds/(365.25*86400)+1970));
-      withinLimits = false;
-    }
-  }
-  if ( withinLimits ) {
-    time_t lsec = (time_t)seconds;
-    fprintf(stderr, "this is: %s\n", ctime(&lsec));
-  }
-#endif
 
-  return Number(seconds == -1 ? NaN : seconds * 1000.0);
+  return seconds == invalidDate ? NaN : seconds * 1000.0;
 }
 
 ///// Awful duplication from krfcdate.cpp - we don't link to kdecore
@@ -571,12 +748,14 @@ static double ymdhms_to_seconds(int year, int mon, int day, int hour, int minute
     return ret;
 }
 
-static const char haystack[37]="janfebmaraprmayjunjulaugsepoctnovdec";
-
 // we follow the recommendation of rfc2822 to consider all
 // obsolete time zones not listed here equivalent to "-0000"
 static const struct {
+#ifdef _WIN32
+    char tzName[4];
+#else
     const char tzName[4];
+#endif
     int tzOffset;
 } known_zones[] = {
     { "UT", 0 },
@@ -592,19 +771,75 @@ static const struct {
     { { 0, 0, 0, 0 }, 0 }
 };
 
-int KJS::local_timeoffset()
+double KJS::makeTime(struct tm *t, double ms, bool utc)
 {
-     static int local_offset = -1;
+    int utcOffset;
+    if (utc) {
+	time_t zero = 0;
+#if defined BSD || defined(__linux__) || defined(__APPLE__)
+	struct tm t3;
+       	localtime_r(&zero, &t3);
+        utcOffset = t3.tm_gmtoff;
+        t->tm_isdst = t3.tm_isdst;
+#else
+        (void)localtime(&zero);
+#  if defined(__BORLANDC__)
+        utcOffset = - _timezone;
+#  else
+        utcOffset = - timezone;
+#  endif
+        t->tm_isdst = 0;
+#endif
+    } else {
+        utcOffset = 0;
+        t->tm_isdst = -1;
+    }
 
-     if ( local_offset != -1 ) return local_offset;
+    double yearOffset = 0.0;
+    if (t->tm_year < (1970 - 1900) || t->tm_year > (2038 - 1900)) {
+      // we'll fool mktime() into believing that this year is within
+      // its normal, portable range (1970-2038) by setting tm_year to
+      // 2000 or 2001 and adding the difference in milliseconds later.
+      // choice between offset will depend on whether the year is a
+      // leap year or not.
+      int y = t->tm_year + 1900;
+      int baseYear = daysInYear(y) == 365 ? 2001 : 2000;
+      const double baseTime = timeFromYear(baseYear);
+      yearOffset = timeFromYear(y) - baseTime;
+      t->tm_year = baseYear - 1900;
+    }
 
-     time_t local = time(0);
-     struct tm* tm_local = gmtime(&local);
-     local_offset = local-mktime(tm_local);
-     if(tm_local->tm_isdst)
-       local_offset += 3600;
+    // Determine if we passed over a DST change boundary
+    if (!utc) {
+      time_t tval = mktime(t) + utcOffset + int((ms + yearOffset)/1000);
+      struct tm t3;
+      localtime_r(&tval, &t3);
+      t->tm_isdst = t3.tm_isdst;
+    }
 
-     return local_offset;
+    return (mktime(t) + utcOffset) * 1000.0 + ms + yearOffset;
+}
+
+// returns 0-11 (Jan-Dec); -1 on failure
+static int findMonth(const char *monthStr)
+{
+  assert(monthStr);
+  static const char haystack[37] = "janfebmaraprmayjunjulaugsepoctnovdec";
+  char needle[4];
+  for (int i = 0; i < 3; ++i) {
+    if (!*monthStr)
+      return -1;
+    needle[i] = tolower(*monthStr++);
+  }
+  needle[3] = '\0';
+  const char *str = strstr(haystack, needle);
+  if (str) {
+    int position = str - haystack;
+    if (position % 3 == 0) {
+      return position / 3;
+    }
+  }
+  return -1;
 }
 
 double KJS::KRFCDate_parseDate(const UString &_date)
@@ -629,7 +864,6 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      char *newPosStr;
      const char *dateString = _date.ascii();
      int day = 0;
-     char monthStr[4];
      int month = -1; // not set yet
      int year = 0;
      int hour = 0;
@@ -647,18 +881,7 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      {
         if ( isspace(*dateString) && dateString - wordStart >= 3 )
         {
-          monthStr[0] = tolower(*wordStart++);
-          monthStr[1] = tolower(*wordStart++);
-          monthStr[2] = tolower(*wordStart++);
-          monthStr[3] = '\0';
-          //fprintf(stderr,"KJS::parseDate found word starting with '%s'\n", monthStr);
-          const char *str = strstr(haystack, monthStr);
-          if (str) {
-            int position = str - haystack;
-            if (position % 3 == 0) {
-              month = position / 3; // Jan=00, Feb=01, Mar=02, ..
-            }
-          }
+          month = findMonth(wordStart);
           while(*dateString && isspace(*dateString))
              dateString++;
           wordStart = dateString;
@@ -666,6 +889,11 @@ double KJS::KRFCDate_parseDate(const UString &_date)
         else
            dateString++;
      }
+     // missing delimiter between month and day (like "January29")?
+     if (month == -1 && dateString && wordStart != dateString) {
+       month = findMonth(wordStart);
+       // TODO: emit warning about dubious format found
+     }
 
      while(*dateString && isspace(*dateString))
      	dateString++;
@@ -674,20 +902,45 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      	return invalidDate;
 
      // ' 09-Nov-99 23:12:40 GMT'
+     errno = 0;
      day = strtol(dateString, &newPosStr, 10);
+     if (errno)
+       return invalidDate;
      dateString = newPosStr;
 
-     if ((day < 1) || (day > 31))
-     	return invalidDate;
      if (!*dateString)
      	return invalidDate;
 
-     if (*dateString == '/' && day <= 12 && month == -1)
+     if (day < 1)
+       return invalidDate;
+     if (day > 31) {
+       // ### where is the boundary and what happens below?
+       if (*dateString == '/' && day >= 1000) {
+         // looks like a YYYY/MM/DD date
+         if (!*++dateString)
+           return invalidDate;
+         year = day;
+         month = strtol(dateString, &newPosStr, 10) - 1;
+         if (errno)
+           return invalidDate;
+         dateString = newPosStr;
+         if (*dateString++ != '/' || !*dateString)
+           return invalidDate;
+         day = strtol(dateString, &newPosStr, 10);
+         if (errno)
+           return invalidDate;
+         dateString = newPosStr;
+       } else {
+         return invalidDate;
+       }
+     } else if (*dateString == '/' && day <= 12 && month == -1)
      {
      	dateString++;
         // This looks like a MM/DD/YYYY date, not an RFC date.....
         month = day - 1; // 0-based
         day = strtol(dateString, &newPosStr, 10);
+        if (errno)
+          return invalidDate;
         dateString = newPosStr;
         if (*dateString == '/')
           dateString++;
@@ -708,22 +961,8 @@ double KJS::KRFCDate_parseDate(const UString &_date)
 
        if ( month == -1 ) // not found yet
        {
-         for(int i=0; i < 3;i++)
-         {
-           if (!*dateString || (*dateString == '-') || isspace(*dateString))
-             return invalidDate;
-           monthStr[i] = tolower(*dateString++);
-         }
-         monthStr[3] = '\0';
-
-         newPosStr = (char*)strstr(haystack, monthStr);
-
-         if (!newPosStr || (newPosStr - haystack) % 3 != 0)
-           return invalidDate;
-
-         month = (newPosStr-haystack)/3; // Jan=00, Feb=01, Mar=02, ..
-
-         if ((month < 0) || (month > 11))
+         month = findMonth(dateString);
+         if (month == -1)
            return invalidDate;
 
          while(*dateString && (*dateString != '-') && !isspace(*dateString))
@@ -743,7 +982,11 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      }
 
      // '99 23:12:40 GMT'
-     year = strtol(dateString, &newPosStr, 10);
+     if (year <= 0 && *dateString) {
+       year = strtol(dateString, &newPosStr, 10);
+       if (errno)
+         return invalidDate;
+     }
 
      // Don't fail if the time is missing.
      if (*newPosStr)
@@ -757,47 +1000,57 @@ double KJS::KRFCDate_parseDate(const UString &_date)
         } else // in the normal case (we parsed the year), advance to the next number
             dateString = ++newPosStr;
 
-        have_time = true;
         hour = strtol(dateString, &newPosStr, 10);
-        dateString = newPosStr;
 
-        if ((hour < 0) || (hour > 23))
-           return invalidDate;
+        // Do not check for errno here since we want to continue
+        // even if errno was set becasue we are still looking
+        // for the timezone!
+        // read a number? if not this might be a timezone name
+        if (newPosStr != dateString) {
+          have_time = true;
+          dateString = newPosStr;
 
-        if (!*dateString)
-           return invalidDate;
+          if ((hour < 0) || (hour > 23))
+            return invalidDate;
 
-        // ':12:40 GMT'
-        if (*dateString++ != ':')
-           return invalidDate;
+          if (!*dateString)
+            return invalidDate;
 
-        minute = strtol(dateString, &newPosStr, 10);
-        dateString = newPosStr;
+          // ':12:40 GMT'
+          if (*dateString++ != ':')
+            return invalidDate;
 
-        if ((minute < 0) || (minute > 59))
-           return invalidDate;
+          minute = strtol(dateString, &newPosStr, 10);
+          if (errno)
+            return invalidDate;
+          dateString = newPosStr;
 
-        // ':40 GMT'
-        if (*dateString && *dateString != ':' && !isspace(*dateString))
-           return invalidDate;
+          if ((minute < 0) || (minute > 59))
+            return invalidDate;
 
-        // seconds are optional in rfc822 + rfc2822
-        if (*dateString ==':') {
-           dateString++;
+          // ':40 GMT'
+          if (*dateString && *dateString != ':' && !isspace(*dateString))
+            return invalidDate;
 
-           second = strtol(dateString, &newPosStr, 10);
-           dateString = newPosStr;
+          // seconds are optional in rfc822 + rfc2822
+          if (*dateString ==':') {
+            dateString++;
 
-           if ((second < 0) || (second > 59))
+            second = strtol(dateString, &newPosStr, 10);
+            if (errno)
               return invalidDate;
+            dateString = newPosStr;
+
+            if ((second < 0) || (second > 59))
+              return invalidDate;
+          }
+
+          while(*dateString && isspace(*dateString))
+            dateString++;
         }
-
-        while(*dateString && isspace(*dateString))
-           dateString++;
-     }
-     else
+     } else {
        dateString = newPosStr;
-
+     }
 
      // don't fail if the time zone is missing, some
      // broken mail-/news-clients omit the time zone
@@ -818,6 +1071,8 @@ double KJS::KRFCDate_parseDate(const UString &_date)
        }
        if ((*dateString == '+') || (*dateString == '-')) {
          offset = strtol(dateString, &newPosStr, 10);
+         if (errno)
+           return invalidDate;
          dateString = newPosStr;
 
          if ((offset < -9959) || (offset > 9959))
@@ -827,6 +1082,8 @@ double KJS::KRFCDate_parseDate(const UString &_date)
          offset = abs(offset);
          if ( *dateString == ':' ) { // GMT+05:00
            int offset2 = strtol(dateString, &newPosStr, 10);
+           if (errno)
+             return invalidDate;
            dateString = newPosStr;
            offset = (offset*60 + offset2)*sgn;
          }
@@ -837,6 +1094,7 @@ double KJS::KRFCDate_parseDate(const UString &_date)
          for (int i=0; known_zones[i].tzName != 0; i++) {
            if (0 == strncasecmp(dateString, known_zones[i].tzName, strlen(known_zones[i].tzName))) {
              offset = known_zones[i].tzOffset;
+             dateString += strlen(known_zones[i].tzName);
              have_tz = true;
              break;
            }
@@ -849,8 +1107,20 @@ double KJS::KRFCDate_parseDate(const UString &_date)
 
      if ( *dateString && year == -1 ) {
        year = strtol(dateString, &newPosStr, 10);
+       if (errno)
+         return invalidDate;
+       dateString = newPosStr;
      }
 
+     while (isspace(*dateString))
+       dateString++;
+
+#if 0
+     // Trailing garbage
+     if (*dateString != '\0')
+       return invalidDate;
+#endif
+
      // Y2K: Solve 2 digit years
      if ((year >= 0) && (year < 50))
          year += 2000;
@@ -858,10 +1128,7 @@ double KJS::KRFCDate_parseDate(const UString &_date)
      if ((year >= 50) && (year < 100))
          year += 1900;  // Y2K
 
-     if ((year < 1900) || (year > 2500))
-     	return invalidDate;
-
-     if (!have_time && !have_tz) {
+     if (!have_tz) {
        // fall back to midnight, local timezone
        struct tm t;
        memset(&t, 0, sizeof(tm));
@@ -869,34 +1136,28 @@ double KJS::KRFCDate_parseDate(const UString &_date)
        t.tm_mon = month;
        t.tm_year = year - 1900;
        t.tm_isdst = -1;
-       return mktime(&t);
+       if (have_time) {
+         t.tm_sec = second;
+         t.tm_min = minute;
+         t.tm_hour = hour;
+       }
+
+       // better not use mktime() as it can't handle the full year range
+       return makeTime(&t, 0, false) / 1000.0;
      }
 
-     if(!have_tz)
-       offset = local_timeoffset();
-     else
-       offset *= 60;
-
-     result = ymdhms_to_seconds(year, month+1, day, hour, minute, second);
-
-     // avoid negative time values
-     if ((offset > 0) && (offset > result))
-        offset = 0;
-
-     result -= offset;
-
-     // If epoch 0 return epoch +1 which is Thu, 01-Jan-70 00:00:01 GMT
-     // This is so that parse error and valid epoch 0 return values won't
-     // be the same for sensitive applications...
-     if (result < 1) result = 1;
-
+     result = ymdhms_to_seconds(year, month+1, day, hour, minute, second) - offset*60;
      return result;
 }
 
 
-Value KJS::timeClip(const Value &t)
+double KJS::timeClip(double t)
 {
-  /* TODO */
-  return t;
+  if (isInf(t))
+    return NaN;
+  double at = fabs(t);
+  if (at > 8.64E15)
+    return NaN;
+  return floor(at) * (t != at ? -1 : 1);
 }
 
