@@ -1,5 +1,5 @@
 /*
- $Id: kthemestyle.cpp,v 1.45 2004/07/12 23:33:08 orlovich Exp $
+ $Id: kthemestyle.cpp 389620 2005-02-15 13:26:54Z hasso $
 
  This file is part of the KDE libraries
  Copyright (C) 1999 Daniel M. Duley <mosfet@kde.org>
@@ -158,7 +158,7 @@ public:
     }
 };
 
-Q_EXPORT_PLUGIN( KThemeStylePlugin )
+KDE_Q_EXPORT_PLUGIN( KThemeStylePlugin )
 
 
 void kDrawWindowsArrow ( QPainter *p, const QStyle* style, QStyle::PrimitiveElement pe, bool down,
@@ -235,7 +235,7 @@ QSize KThemeStyle::sizeFromContents( ContentsType contents,
                 // Ensure we stick to standard width and heights.
                 if ( button->isDefault() || button->autoDefault() )
                 {
-                    if ( w < 80 && !button->pixmap() )
+                    if ( w < 80 && !button->text().isEmpty() )
                         w = 80;
                 }
 
